# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com

'''
This test case class is intended to help test functions and features related to block device mapping for ebs backed
instances. The test attempts to create a base ebs-backed image created from a remote url downloaded and written to a
volume within this cloud. A snapshot of this volume, and md5sum are taken for use throughout the different test
iterations. In addition a 2nd volume is filled with some amount of random/unique data before a 2nd snapshot and md5sum
of it are created and used through the test as well.
This test class attempts to test the different block device attributes and verify them through the different stages
of the vm/guest.



Some notes/examples on euca2ools registration using block dev mapping:
ec2-register -n ImageName --root-device-name /dev/sda1 -s snap-e1eb279f
-b "/dev/sdb=ephemeral0" -b "/dev/sdh=snap-d5eb27ab" -b "/dev/sdj=:100"

This command maps /dev/sda to a EBS root volume based on a snapshot, /dev/sdb to ephemeral0,
/dev/sdh to an EBS volume based on a snapshot,
and /dev/sdj to an empty EBS volume that is 100 GiB in size. The output is the ID for your new AMI.

'''

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase

from boto.ec2.blockdevicemapping import BlockDeviceMapping, BlockDeviceType
import time
import types
import httplib
import copy
import math
import re


class FixedBlockDeviceMapping(BlockDeviceMapping):
    #Temporary class until boto fixes this upstream.
    # 'nodevice' param is formatted incorrectly, should not prefix with 'Ebs' before 'NoDevice'.

    def build_list_params(self, params, prefix=''):
        i = 1
        for dev_name in self:
            pre = '%sBlockDeviceMapping.%d' % (prefix, i)
            params['%s.DeviceName' % pre] = dev_name
            block_dev = self[dev_name]
            if block_dev.ephemeral_name:
                params['%s.VirtualName' % pre] = block_dev.ephemeral_name
            else:
                if block_dev.no_device:
                    params['%s.NoDevice' % pre] = 'true'
                if block_dev.snapshot_id:
                    params['%s.Ebs.SnapshotId' % pre] = block_dev.snapshot_id
                if block_dev.size:
                    params['%s.Ebs.VolumeSize' % pre] = block_dev.size
                if block_dev.delete_on_termination:
                    params['%s.Ebs.DeleteOnTermination' % pre] = 'true'
                else:
                    params['%s.Ebs.DeleteOnTermination' % pre] = 'false'
            i += 1




class Block_Device_Mapping_Tests(EutesterTestCase):
    #Define the bytes per gig
    gig = 1073741824

    def __init__(self, url=None, tester=None, instance_password=None, **kwargs):
        #### Pre-conditions
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument('--use_previous',
                                 action='store_true', default=False,
                                 help='Use existing volumes, snapshots and images from a previous Block_Device_Mapping_Test run')
        self.parser.add_argument("--url",
                                 dest="url",
                                 help="URL containing remote BFEBS image to create emi from",
                                 default=None)
        self.tester = tester
        self.url = url
        self.get_args()
        # Allow __init__ to get args from __init__'s kwargs or through command line parser...
        for kw in kwargs:
            print 'Setting kwarg:'+str(kw)+" to "+str(kwargs[kw])
            self.set_arg(kw ,kwargs[kw])
        self.show_args()
        #if self.args.config:
        #    setattr(self.args, 'config_file',self.args.config)
        # Setup basic eutester object
        if not self.tester:
            try:
                self.tester = self.do_with_args(Eucaops)
            except Exception, e:
                raise Exception('Couldnt create Eucaops tester object, make sure credpath, ' \
                                'or config_file and password was provided, err:' + str(e))
            #replace default eutester debugger with eutestcase's for more verbosity...
            self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
        if not self.url:
            try:
                self.url = self.args.url

            except Exception, e:
                raise Exception('')
        self.reservation = None
        self.instance = None
        self.volumes = []
        self.image_bytes = None
        self.image_gigs = None
        self.build_image_volume = None
        self.build_image_volume_tag_name = 'block_dev_map_tests_build_image_volume'
        self.build_image_snapshot = None
        self.build_image_snapshot_tag_name = 'block_dev_map_tests_build_image_snapshot'
        self.block_device_map = None
        self.base_test_volume = None
        self.base_test_volume_tag_name = 'block_dev_map_tests_base_test_volume'
        self.base_test_snapshot = None
        self.base_test_snapshot_tag_name = 'block_dev_map_tests_base_test_snap'
        self.test_image1 = None
        self.test_image1_tag_name = 'block_dev_map_tests_image1'
        self.test_image2 = None
        self.test_image2_tag_name = 'block_dev_map_tests_image2'
        self.test_image3 = None
        self.test_image3_tag_name = 'block_dev_map_tests_image3'
        self.test_image4 = None
        self.test_image4_tag_name = 'block_dev_map_tests_image4'
        self.snapshots = []
        self.images = []
        self.instance_password =instance_password

        ### Add and authorize a group for the instance
        if self.args.zone:
            self.zone = str(self.args.zone)
        else:
            self.zone = 'PARTI00'
        self.groupname = 'jenkins'
        self.group = self.tester.add_group(self.groupname)
        self.tester.authorize_group(self.group)
        self.tester.authorize_group(self.group, protocol='icmp',port='-1')
        ### Generate a keypair for the instance
        if self.instance_password:
            self.keypair = None
        else:
            try:
                keys = self.tester.get_all_current_local_keys()
                if keys:
                    self.keypair = keys[0]
                else:
                    self.keypair = self.tester.add_keypair('mpathtestinstancekey'+str(time.time()))
            except Exception, ke:
                raise Exception("Failed to find/create a keypair, error:" + str(ke))

        ### Get an image to work with
        if self.args.emi:
            self.image = self.tester.get_emi(emi=str(self.args.emi))
        else:
            self.image = self.tester.get_emi(root_device_type="instance-store",not_location='windows')
        if not self.image:
            raise Exception('couldnt find instance store image')
        self.clean_method = self.cleanup


    def cleanup(self):
        '''
        Definition:
        Clean up test artifacts leaving behind the images and snapshots created during this test
        '''
        #leave the base test, and snapshot behind for future use. To remove these,
        # the images they are associated with will need to be deleted as well.
        id_list = []
        delete_list = []
        for snap in self.tester.test_resources['snapshots']:
            if self.base_test_snapshot and re.search( snap.id, self.base_test_snapshot.id):
                self.debug('Removing base_test_snapshot from test_resources list so not deleted')
            elif self.build_image_snapshot and re.search(snap.id, self.build_image_snapshot.id):
                self.debug('Removing build_image_snapshot from test_resources list so not deleted')
            else:
                delete_list.append(snap)
        self.tester.test_resources['snapshots'] = delete_list
        self.tester.cleanup_artifacts()


    def get_remote_file_size_via_http(self, url=None):
        url = url or self.url
        #Get the remote file size from the http header of the url given
        try:
            url = url.replace('http://','')
            host = url.split('/')[0]
            path = url.replace(host,'')
            self.debug("get_remote_file, host("+host+") path("+path+")")
            conn=httplib.HTTPConnection(host)
            conn.request("HEAD", path)
            res=conn.getresponse()
            image_bytes = int(res.getheader('content-length'))
            self.debug("content-length:"+str(self.image_bytes))
            conn.close()
        except Exception, e:
            self.debug("Failed to get remote file size...")
            raise e
        self.status('URL:' + str(url) + "\ncontent-length:" + str(image_bytes))
        return image_bytes


    def get_existing_test_snapshot_by_tag_key(self, tagkey):
        snapshots = self.tester.get_snapshots(filters={'tag-key':str(tagkey)})
        for snapshot in snapshots:
            if 'md5' in snapshot.tags and 'md5len' in snapshot.tags:
                return snapshot
        return None

    def get_existing_test_volume_by_tag_key(self, tagkey):
        volumes = self.tester.get_volumes(filters={'tag-key':str(tagkey)})
        for volume in volumes:
            if 'md5' in volume.tags and 'md5len' in volume.tags:
                return volume
        return None

    def get_existing_test_image_by_tag_key(self, tagkey):
        try:
            return self.tester.get_emi(filters={'tag-key':str(tagkey)})
        except:
            return None




    def setup_bfebs_instance_volume_and_snapshots_from_url(self, url=None, create_test_vol=True, time_per_gig=100):

        url = url or self.url
        volumes = []
        self.image_bytes = self.get_remote_file_size_via_http(url=url)
        self.image_gigs = int(math.ceil(float(self.image_bytes)/self.gig) or 1)
        curl_timeout= self.image_gigs * time_per_gig
        self.status('Attempting to launch instance store instance...')
        instance = self.tester.run_image(self.image,
                                              keypair=self.keypair.name,
                                              type=self.args.vmtype,
                                              group=self.group.name,
                                              zone=self.zone)[0]
        self.current_test_instance = instance
        self.status('create test volume(s)...')
        self.build_image_volume = self.tester.create_volumes(self.zone, size = self.image_gigs,monitor_to_state=None)[0]
        volumes.append(self.build_image_volume)
        self.build_image_volume.add_tag(self.build_image_volume_tag_name)

        if create_test_vol:
            self.base_test_volume = self.tester.create_volumes(self.zone, size = 1, monitor_to_state=None )[0]
            volumes.append(self.base_test_volume)
            self.base_test_volume.add_tag(self.base_test_volume_tag_name)
        self.tester.monitor_created_euvolumes_to_state(volumes=volumes)
        self.status('Copy the remote bfebs image into a volume and create snapshot from it...')
        instance.attach_volume(self.build_image_volume)
        instance.sys("curl "+url+" > "+ self.build_image_volume.guestdev+" && sync", timeout=curl_timeout)
        instance.md5_attached_euvolume(self.build_image_volume)
        self.build_image_snapshot = self.tester.create_snapshots(volume=self.build_image_volume)[0]
        #update test resources with tags...
        self.build_image_snapshot.add_tag(self.build_image_snapshot_tag_name)
        self.build_image_snapshot.add_tag('md5',self.build_image_volume.md5)
        self.build_image_snapshot.add_tag('md5len',self.build_image_volume.md5len)
        self.build_image_snapshot.add_tag('src_url', str(url))
        self.build_image_volume.add_tag('src_url', str(url))
        self.build_image_volume.add_tag('md5', self.build_image_volume.md5)
        self.build_image_volume.add_tag('md5len', self.build_image_volume.md5len)

        self.status('Done creating BFEBS build_image_snapshot and volume and md5ing volume')
        if create_test_vol:
            self.status('Attaching test volume, writing random data into it and gathering md5...')
            instance.attach_volume(self.base_test_volume)
            instance.vol_write_random_data_get_md5(self.base_test_volume, overwrite=True)
            self.base_test_snapshot = self.tester.create_snapshots(volume=self.base_test_volume)[0]
            self.base_test_snapshot.add_tag(self.base_test_snapshot_tag_name)
            self.base_test_volume.add_tag('md5', self.base_test_volume.md5)
            self.base_test_volume.add_tag('md5len', self.base_test_volume.md5len)
            self.base_test_snapshot.add_tag('md5', self.base_test_volume.md5)
            self.base_test_snapshot.add_tag('md5len', self.base_test_volume.md5len)
        instance.terminate_and_verify()
        return self.build_image_snapshot


    def add_block_device_types_to_mapping(self,
                                        device_name,
                                        ephemeral_name=None,
                                        snapshot_id=None,
                                        size=None,
                                        no_device=False,
                                        delete_on_terminate=True,
                                        block_device_map=None):
        block_device_map = block_device_map or FixedBlockDeviceMapping()
        block_dev_type = BlockDeviceType()
        block_dev_type.delete_on_termination = delete_on_terminate
        block_dev_type.no_device = no_device
        block_dev_type.size = size

        if snapshot_id:
            if isinstance(snapshot_id, types.StringTypes):
                snapshot_id = snapshot_id
            else:
                snapshot_id = snapshot_id.id
            block_dev_type.snapshot_id = snapshot_id
        elif ephemeral_name:
            block_dev_type.ephemeral_name = ephemeral_name

        block_device_map[device_name] = block_dev_type
        return block_device_map

    def create_bfebs_image(self,
                           snapshot,
                           root_device_name = '/dev/sda',
                           name=None,
                           delete_on_terminate=True,
                           size=None,
                           block_device_map=None):
        if isinstance(snapshot, types.StringTypes):
            snapshot_id = snapshot
        else:
            snapshot_id = snapshot.id
        image_id = self.tester.register_snapshot_by_id(snap_id=snapshot_id,
                                                        root_device_name=root_device_name,
                                                        size=size,
                                                        dot=delete_on_terminate,
                                                        name=name,
                                                        block_device_map=block_device_map
                                                        )
        new_image = self.tester.get_emi(emi=image_id)
        self.images.append(new_image)
        return new_image


    def populate_self_from_existing_test_resources(self,build_image=True, base_test=True):
        if build_image:
            self.build_image_snapshot = self.get_existing_test_snapshot_by_tag_key(tagkey=self.build_image_snapshot_tag_name)
            if self.build_image_snapshot:
                self.build_image_snapshot.eutest_volume_md5 = self.build_image_snapshot.tags.get('md5')
                self.build_image_snapshot.eutest_volume_md5len = int(self.build_image_snapshot.tags.get('md5len'))
            self.build_image_volume = self.get_existing_test_volume_by_tag_key(tagkey=self.build_image_volume_tag_name)
            if self.build_image_volume:
                self.build_image_volume.md5 = self.build_image_volume.tags.get('md5')
                self.build_image_volume.md5len = int(self.build_image_volume.tags.get('md5len'))
        if base_test:
            self.base_test_snapshot = self.get_existing_test_snapshot_by_tag_key(tagkey=self.base_test_snapshot_tag_name)
            if self.base_test_snapshot:
                self.base_test_snapshot.eutest_volume_md5 = self.base_test_snapshot.tags.get('md5')
                self.base_test_snapshot.eutest_volume_md5len = int(self.base_test_snapshot.tags.get('md5len'))
            self.base_test_volume = self.get_existing_test_volume_by_tag_key(self.base_test_volume_tag_name)
            if self.base_test_volume:
                self.base_test_volume.md5 = self.base_test_volume.tags.get('md5')
                self.base_test_volume.md5len = int(self.base_test_volume.tags.get('md5len'))
        self.test_image1 = self.get_existing_test_image_by_tag_key(self.test_image1_tag_name)
        self.test_image2 = self.get_existing_test_image_by_tag_key(self.test_image2_tag_name)
        self.test_image3 = self.get_existing_test_image_by_tag_key(self.test_image3_tag_name)
        self.test_image4 = self.get_existing_test_image_by_tag_key(self.test_image4_tag_name)


    def try_existing_resources_else_create_them(self, url=None):

        self.populate_self_from_existing_test_resources()
        if not self.build_image_snapshot or not self.base_test_snapshot:
            self.debug('\nPopulate instances from existing missing all or some of the required resources:' +
                       '\nbuild_image_volume:' + str(self.build_image_volume) +
                       '\nbuild_image_snapshot(required):' + str(self.build_image_snapshot) +
                       '\nbase_test_snapshot(required):' + str(self.base_test_snapshot) +
                       '\nbase_test_volume:' + str(self.base_test_volume) + "\n")
            self.debug('Building missing resources now...')
            url = url or self.url
            if not url:
                raise Exception('URL to bfebs image is needed to build snapshot test resource')
            self.setup_bfebs_instance_volume_and_snapshots_from_url(url=url)
        if not self.build_image_volume:
            self.build_test_volume = self.tester.create_volume(zone=self.zone, snapshot=self.build_image_snapshot)
            self.build_image_volume.md5 = self.build_image_snapshot.md5=self.build_image_snapshot.eutest_volume_md5
            self.build_image_volume.md5len = self.build_image_snapshot.eutest_volume_md5len
            self.build_image_volume.add_tag('md5', self.build_image_volume.md5)
            self.build_image_volume.add_tag('md5len', self.build_image_volume.md5len)
        if not self.base_test_volume:
            self.status('Creating base_test_volume from base_test_snapshot: ')
            self.base_test_volume = self.tester.create_volume(zone=self.zone, snapshot=self.base_test_snapshot)
            self.base_test_volume.md5 = self.base_test_snapshot.md5=self.build_image_snapshot.eutest_volume_md5
            self.base_test_volume.md5len = self.base_test_snapshot.eutest_volume_md5len
            self.base_test_volume.add_tag('md5', self.base_test_volume.md5)
            self.base_test_volume.add_tag('md5len', self.base_test_volume.md5len)

    def find_volume_on_euinstance(self, euinstance, map_device_name, md5=None, md5len=None, euvolume=None, ):
        '''
        Find block devices on guest by the MD5 sum of the volume(s) they were created from.
        Will attempt to find euvolumes on an instance 'if' the euvolume(s) have md5sums to use for
        comparison on the guest's dev dir.
        '''
        md5 = md5 or euvolume.md5
        md5len = md5len or euvolume.md5len
        return euinstance.get_guest_dev_for_block_device_map_device(md5=md5,md5len=md5len,map_device=map_device_name)

        #return euinstance.find_blockdev_by_md5(md5=euvolume.md5, md5len=euvolume.md5len)


    def register_n_run_test1_bfebs_image_w_dot_is_true(self):
        '''
        Will test running a default bfebs image. Additional tests here:
        -will verify a portion of the root device's md5sum against the original volume's/snapshot's
        -will verify delete on terminate set to True, will expect backing volume to be deleted
        -will attempt to verify volume states through stopping and restarting the instance
        -will attempt to verify instance meta data for the block device map in use
        -will look for 'no' ephemeral
        '''
        errmsg = ""
        image_name = self.test_image1_tag_name + "_" + str(self.build_image_snapshot.id)
        image = self.create_bfebs_image(snapshot=self.build_image_snapshot, name=image_name, delete_on_terminate=True)
        if not image.block_device_mapping.get(image.root_device_name).delete_on_termination:
            raise Exception('Expected DOT is True, instead image delete on termination set to:' +
                            str(image.block_device_mapping.get(image.root_device_name.delete_on_termination)))
        self.status('Created image:'+str(image.id)+', now running instance from it...')
        self.tester.print_block_device_map(image.block_device_mapping)
        self.test_image1 = image
        self.test_image1.add_tag(self.test_image1_tag_name)
        instance = self.tester.run_image(image=image, keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance devices for md5sums which match original volume/snapshots...')
            self.find_volume_on_euinstance(instance,
                                           instance.root_device_name,
                                           md5=self.build_image_snapshot.eutest_volume_md5,
                                           md5len=self.build_image_snapshot.eutest_volume_md5len)
            self.status('Checking instance for to make sure ephemeral is not present...')
            try:
                instance.get_ephemeral_dev()
            except:
                self.debug('Ephemeral was not found, passing...')
            else:
                raise Exception('Ephemeral device found, but none provided?')
            self.status('Check block device mapping meta data...')
            instance.check_instance_meta_data_for_block_device_mapping()
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)


    def register_n_run_test2_bfebs_image_w_dot_is_false(self):
        '''
        Will test running a default bfebs image. Additional tests here:
        -will verify a portion of the root device's md5sum against the original volume's/snapshot's
        -will verify delete on terminate set to True, will expect backing volume to remain and become available.
        -will look for 'no' ephemeral
        '''
        errmsg = ""
        image_name = self.test_image2_tag_name + "_" + str(self.build_image_snapshot.id)
        image = self.create_bfebs_image(snapshot=self.build_image_snapshot, name=image_name, delete_on_terminate=False)
        if image.block_device_mapping.get(image.root_device_name).delete_on_termination:
            raise Exception('Expected DOT is False, instead image delete on termination set to:' +
                            str(image.block_device_mapping.get(image.root_device_name.delete_on_termination)))
        self.status('Created image:'+str(image.id)+', now running instance from it...')
        self.tester.print_block_device_map(image.block_device_mapping)
        self.test_image2 = image
        self.test_image2.add_tag(self.test_image2_tag_name)
        instance = self.tester.run_image(image=image, keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance devices for md5sums which match original volume/snapshots...')
            self.find_volume_on_euinstance(instance,
                                           instance.root_device_name,
                                           md5=self.build_image_snapshot.eutest_volume_md5,
                                           md5len=self.build_image_snapshot.eutest_volume_md5len)
            self.status('Checking instance for to make sure ephemeral is not present...')
            try:
                instance.get_ephemeral_dev()
            except:
                self.debug('Ephemeral was not found, passing...')
            else:
                raise Exception('Ephemeral device found, but none provided?')
            self.status('Check block device mapping meta data...')
            instance.check_instance_meta_data_for_block_device_mapping()
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)


    def register_n_run_test3_bfebs_image_w_ephemeral_map_snap_map_and_ebsvol_map_dot_is_true(self):
        '''
        Will test running a default bfebs image registered w/ the following mapped devices, delete on terminate for
        all devices is true:
        1)the snapshot containing the original bfebs image
        2)the base test snapshot created by this testcase
        3)an empty volume
        Additional tests here:
        -will verify a portion of the root device's md5sum against the original volume's/snapshot's
        -will verify a portion of the test snapshot's md5sum against the guest's device
        -will verify the presence and size of the ephemeral disk
        -will attempt to verify the presence and size of empty volume's device (by process of elimination)
        -will attempt to verify the empty volume's requested size against the size of the block dev on guest
        -will attempt to verify volume states through stopping and restarting the instance
        -will attempt to verify instance meta data for the block device map in use
        -will verify delete on terminate set to True, and all volumes are deleted post instance termination
        '''
        errmsg = ""
        image_name = self.test_image3_tag_name + "_" + str(self.build_image_snapshot.id)
        bdm_emptyvol_dev = '/dev/vdd'
        bdm_snapshot_dev = '/dev/vdc'
        bdm_ephemeral_dev = '/dev/vdb'
        bdm_ephemeral_name = 'ephemeral0'
        bdm_emptyvol_size = 2

        self.status('Creating image with block device mapping...')
        bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                               device_name=bdm_snapshot_dev,
                                               delete_on_terminate=True)
        self.add_block_device_types_to_mapping(size = bdm_emptyvol_size,
                                               device_name = bdm_emptyvol_dev,
                                               delete_on_terminate=True,
                                               block_device_map=bdm)
        #add ephemeral dev separately to use later for bug work around...
        eph_dev = BlockDeviceType()
        eph_dev.device_name=bdm_ephemeral_dev
        eph_dev.ephemeral_name=bdm_ephemeral_name
        bdm[bdm_ephemeral_dev] = eph_dev
        self.tester.print_block_device_map(bdm)
        image = self.create_bfebs_image(snapshot=self.build_image_snapshot,
                                        block_device_map=bdm,
                                        name=image_name,
                                        delete_on_terminate=True)
        self.tester.print_block_device_map(image.block_device_mapping)
        self.test_image3 = image
        self.test_image3.add_tag(self.test_image3_tag_name)
        self.status('Created image:'+str(image.id)+', now running instance from it...')
        instance = self.tester.run_image(image=image,keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also' +
                         'record the volume id, md5 and guest device within the instance for later stop, start, and detach' +
                         'operations...')
            self.status('Getting guest device for root device...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            self.status('Getting guest device for snapshot device...')
            guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                                md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                                map_device=bdm_snapshot_dev)
            self.status('Checking instance for ephemeral disk and size...')
            guest_ephemeral_dev = instance.check_ephemeral_against_vmtype()
            self.status("Ephemeral verified, Attempting to find guest's device empty volume by process of elimination...")
            remaining_devs=self.find_remaining_devices(instance,[guest_root_dev, guest_snap_dev, guest_ephemeral_dev])
            if len(remaining_devs) != 1:
                raise Exception('Could not find empty vol dev from remaining devs on guest:' + str(",").join(remaining_devs))
            guest_emptyvol_device = '/dev/' + str(remaining_devs[0]).replace('/dev/','')
            self.status('Found empty volume on guest device:'+ str(guest_emptyvol_device)+', checking for proper size...')
            vol_size = instance.get_blockdev_size_in_bytes(guest_emptyvol_device) / self.gig
            if vol_size != bdm_emptyvol_size:
                raise Exception('Block device size on guest:' + str(vol_size) +
                                ", does not match requested size:" + str(bdm_emptyvol_size))
            self.status('Found empty volume device on guest, writing random  data and storing md5 prior to stop/start tests...')
            empty_vol = self.tester.get_volume(instance.block_device_mapping.get(bdm_emptyvol_dev).volume_id)
            empty_vol.guestdev = guest_emptyvol_device
            if not empty_vol in instance.attached_vols:
                instance.attached_vols.append(empty_vol)
            instance.vol_write_random_data_get_md5(empty_vol)
            self.status('Check block device mapping meta data...')
            #Temp work around for existing bug where ephemeral is not reported...
            meta_bdm = instance.block_device_mapping
            if not meta_bdm.has_key(bdm_ephemeral_dev):
                self.resulterr('Ephemeral disk not reported in instance block dev mapping: see euca-6048')
                meta_bdm[bdm_ephemeral_dev] = eph_dev
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=meta_bdm)
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)

    def register_n_run_test4_bfebs_image_w_ephemeral_map_snap_map_and_ebsvol_map_dot_is_false(self):
        '''
        Will test running a default bfebs image registered w/ the following mapped devices, delete on terminate for
        all devices is False:
        1)the snapshot containing the original bfebs image
        2)the base test snapshot created by this testcase
        3)an empty volume
        Additional tests here:
        -will verify a portion of the root device's md5sum against the original volume's/snapshot's
        -will verify a portion of the test snapshot's md5sum against the guest's device
        -will verify the presence and size of the ephemeral disk
        -will attempt to verify the presence and size of empty volume's device (by process of elimination)
        -will attempt to verify the empty volume's requested size against the size of the block dev on guest
        -will attempt to verify volume states through stopping and restarting the instance
        -will attempt to verify instance meta data for the block device map in use
        -will verify delete on terminate set to False, and all volumes are not deleted post instance termination
        '''
        errmsg = ""
        image_name = self.test_image4_tag_name + "_" + str(self.build_image_snapshot.id)
        bdm_emptyvol_dev = '/dev/vdd'
        bdm_snapshot_dev = '/dev/vdc'
        bdm_ephemeral_dev = '/dev/vdb'
        bdm_ephemeral_name = 'ephemeral0'
        bdm_emptyvol_size = 2

        self.status('Creating image with block device mapping...')
        bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                               device_name=bdm_snapshot_dev,
                                               delete_on_terminate=False)
        self.add_block_device_types_to_mapping(device_name = bdm_emptyvol_dev,
                                               size = bdm_emptyvol_size,
                                               delete_on_terminate=False,
                                               block_device_map=bdm)
        #add ephemeral dev separately to use later for bug work around...
        eph_dev = BlockDeviceType()
        eph_dev.device_name=bdm_ephemeral_dev
        eph_dev.ephemeral_name=bdm_ephemeral_name
        bdm[bdm_ephemeral_dev] = eph_dev
        self.tester.print_block_device_map(bdm)
        image = self.create_bfebs_image(snapshot=self.build_image_snapshot,
                                        block_device_map=bdm,
                                        name=image_name,
                                        delete_on_terminate=False)
        self.tester.print_block_device_map(image.block_device_mapping)
        self.test_image4 = image
        self.test_image4.add_tag(self.test_image4_tag_name)
        self.status('Created image:'+str(image.id)+', now running instance from it...')
        instance = self.tester.run_image(image=image, keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also \
                         record the volume id, md5 and guest device within the instance for later stop, start, and detach \
                         operations...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                                md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                                map_device=bdm_snapshot_dev)
            self.status('Checking instance for ephemeral disk and size...')
            guest_ephemeral_dev = instance.check_ephemeral_against_vmtype()
            self.status("Attempting to find guest's device empty volume by process of elimination...")
            remaining_devs=self.find_remaining_devices(instance,[guest_root_dev, guest_snap_dev, guest_ephemeral_dev])
            if len(remaining_devs) != 1:
                raise Exception('Could not find empty vol dev from remaining devs on guest:' + str(",").join(remaining_devs))
            guest_emptyvol_device = '/dev/' + str(remaining_devs[0]).replace('/dev/','')
            self.status('Found empty volume on guest device:'+ str(guest_emptyvol_device)+', checking for proper size...')
            vol_size = instance.get_blockdev_size_in_bytes(guest_emptyvol_device) / self.gig
            if vol_size != bdm_emptyvol_size:
                raise Exception('Block device size on guest:' + str(vol_size) +
                                ", does not match requested size:" + str(bdm_emptyvol_size))
            self.status('Found empty volume device on guest, writing random  data and storing md5 prior to stop/start tests...')
            empty_vol = self.tester.get_volume(instance.block_device_mapping.get(bdm_emptyvol_dev).volume_id)
            empty_vol.guestdev = guest_emptyvol_device
            if not empty_vol in instance.attached_vols:
                instance.attached_vols.append(empty_vol)
            instance.vol_write_random_data_get_md5(empty_vol)

            self.status('Check block device mapping meta data...')
            #Temp work around for existing bug where ephemeral is not reported...
            meta_bdm = instance.block_device_mapping
            if not meta_bdm.has_key(bdm_ephemeral_dev):
                self.resulterr('Ephemeral disk not reported in instance block dev mapping: see euca-6048')
                meta_bdm[bdm_ephemeral_dev] = eph_dev
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=meta_bdm)
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)


    def run_time_test1_image1_overwrite_root_vol_size_and_set_dot_to_false(self):
        '''
        Will test running a default bfebs image registered w/ the following mapped devices, delete on terminate for
        all devices is False:
        1)the snapshot containing the original bfebs image

        At run time the following block device map attributes will be over written by:
        -root device size increased by 1g
        -delete on terminate flag set from True to False

        Additional tests here:
        -will verify a portion of the root device's md5sum against the original volume's/snapshot's
        -will verify that the guest's block dev representing the root dev is the new size provided
        -will verify an ephemeral disk is not provided
        -will attempt to verify volume states through stopping and restarting the instance
        -will attempt to verify instance meta data for the block device map in use
        -will verify delete on terminate set to False, and all volumes are not deleted post instance termination
        '''
        errmsg = ""
        image = self.test_image1
        #Add 1 to current root device size...
        bdm_rootsnap_size = image.block_device_mapping.get(image.root_device_name).size + 1
        #Get opposite of DOT value...
        dot_value = (not image.block_device_mapping.get(image.root_device_name).delete_on_termination)
        self.status('Using image:' +str(self.test_image1.id)+ ", root dev size set to:" +
                     str(bdm_rootsnap_size) + " and dot to false at run time...")
        self.status('Original Image block device map:')
        self.tester.print_block_device_map(image.block_device_mapping)
        bdm = self.add_block_device_types_to_mapping(device_name=image.root_device_name,
                                                     size=bdm_rootsnap_size,
                                                     delete_on_terminate=dot_value)
        self.status('Applying the following block device map:')
        self.tester.print_block_device_map(bdm)
        instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also \
                         record the volume id, md5 and guest device within the instance for later stop, start, and detach \
                         operations...')
            self.status('Checking device(s) size(s) on guest vs requested in bdm...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            self.status('Checking device(s) size(s) on guest vs requested in bdm...')
            root_dev_size = instance.get_blockdev_size_in_bytes(guest_root_dev) / self.gig
            if root_dev_size != bdm_rootsnap_size:
                raise Exception('Root device size on guest:' + str(root_dev_size) +
                                ' !=  requested size' + str(bdm_rootsnap_size) +
                                ", original size in image:" + str(image.block_device_mapping.get(image.root_device_name).size))
            self.status('Checking instance for to make sure ephemeral is not present...')
            try:
                instance.get_ephemeral_dev()
            except:
                self.debug('Ephemeral was not found, passing...')
            else:
                raise Exception('Ephemeral device found, but none provided?')
            self.status('Check block device mapping meta data...')
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=bdm)
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)


    def run_time_test2_image1_add_ephemeral_map_snap_map_emptyvol_map_at_run_time_dot_true(self):
        '''
        Will test running a default bfebs image registered w/ the following mapped devices, delete on terminate for
        all devices is False:
        1)the snapshot containing the original bfebs image

        At run time the following block devices will be applied per map:
        1) Single ephemeral disk, size of vm type disk
        2) Single volume created from the local base_test_snapshot - size altered by +1 gig
        3) Single empty volume

        Additional tests here:
        -will verify a portion of the root device's md5sum against the original volume's/snapshot's
        -will verify a portion of the test snapshot's md5sum against the guest's device and size of disk
        -will verify the presence and size of the ephemeral disk
        -will attempt to verify the presence and size of empty volume's device (by process of elimination)
        -will attempt to verify the empty volume's requested size against the size of the block dev on guest
        -will attempt to verify volume states through stopping and restarting the instance
        -will attempt to verify instance meta data for the block device map in use
        -will verify delete on terminate set to True, and all volumes are deleted post instance termination
        '''
        errmsg = ""
        image = self.test_image1
        bdm_emptyvol_dev = '/dev/vdd'
        bdm_snapshot_dev = '/dev/vdc'
        bdm_ephemeral_dev = '/dev/vdb'
        bdm_ephemeral_name = 'ephemeral0'
        bdm_rootsnap_size = image.block_device_mapping.get(image.root_device_name).size
        bdm_emptyvol_size = 1
        bdm_snap_size = self.base_test_snapshot.volume_size + 1

        self.status('Using image:' +str(self.test_image1.id)+ ", ephemeral, and snapshot ebs devices added at run time..")
        self.status('Original Image block device map:')
        self.tester.print_block_device_map(image.block_device_mapping)
        bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                               size = bdm_snap_size,
                                               device_name=bdm_snapshot_dev,
                                               delete_on_terminate=False)
        self.add_block_device_types_to_mapping(device_name = bdm_emptyvol_dev,
                                               size = bdm_emptyvol_size,
                                               delete_on_terminate=False,
                                               block_device_map=bdm)
        #add ephemeral dev separately to use later for bug work around...
        eph_dev = BlockDeviceType()
        eph_dev.device_name=bdm_ephemeral_dev
        eph_dev.ephemeral_name=bdm_ephemeral_name
        bdm[bdm_ephemeral_dev] = eph_dev
        self.status('Applying the following block device map:')
        self.tester.print_block_device_map(bdm)
        instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance for ephemeral disk and size...')
            guest_ephemeral_dev = instance.check_ephemeral_against_vmtype()
            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also \
                         record the volume id, md5 and guest device within the instance for later stop, start, and detach \
                         operations...')
            self.status('Checking device(s) size(s) on guest vs requested in bdm...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            root_dev_size = instance.get_blockdev_size_in_bytes(guest_root_dev) / self.gig
            if root_dev_size != bdm_rootsnap_size:
                raise Exception('Root device size on guest:' + str(root_dev_size) +
                                ' !=  requested size' + str(bdm_rootsnap_size) +
                                ", original size in image:" + str(image.block_device_mapping.get(image.root_device_name).size))

            #Volume from snapshot checks...
            guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                                md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                                map_device=bdm_snapshot_dev)
            guest_snapvol_size = instance.get_blockdev_size_in_bytes(guest_snap_dev) / self.gig
            if guest_snapvol_size != bdm_snap_size:
                raise Exception('Volume size on guest:'+ str(guest_snapvol_size)+' != ' + str(bdm_snap_size) +
                                ', the size requested for bdm snap:' + str(self.base_test_snapshot.id))

            #Empty vol checks...
            self.status("Attempting to find guest's device empty volume by process of elimination...")
            remaining_devs=self.find_remaining_devices(instance,[guest_root_dev, guest_snap_dev, guest_ephemeral_dev])
            if len(remaining_devs) != 1:
                raise Exception('Could not find empty vol dev from remaining devs on guest:' + str(",").join(remaining_devs))
            guest_emptyvol_device = '/dev/' + str(remaining_devs[0]).replace('/dev/','')
            self.status('Found empty volume on guest device:'+ str(guest_emptyvol_device)+', checking for proper size...')
            vol_size = instance.get_blockdev_size_in_bytes(guest_emptyvol_device) / self.gig
            if vol_size != bdm_emptyvol_size:
                raise Exception('Block device size on guest:' + str(vol_size) +
                                ", does not match requested size:" + str(bdm_emptyvol_size))
            self.status('Found empty volume device on guest, writing random  data and storing md5 prior to stop/start tests...')
            empty_vol = self.tester.get_volume(instance.block_device_mapping.get(bdm_emptyvol_dev).volume_id)
            empty_vol.guestdev = guest_emptyvol_device
            if not empty_vol in instance.attached_vols:
                instance.attached_vols.append(empty_vol)
            instance.vol_write_random_data_get_md5(empty_vol)

            self.status('Check block device mapping meta data...')
            #Temp work around for existing bug where ephemeral is not reported...
            meta_bdm = instance.block_device_mapping
            if not meta_bdm.has_key(bdm_ephemeral_dev):
                self.resulterr('Ephemeral disk not reported in instance block dev mapping: see euca-6048')
                meta_bdm[bdm_ephemeral_dev] = eph_dev
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=meta_bdm)
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)

    def run_time_test3_image3_overwrite_all_non_root_w_no_device(self):
        errmsg = ""
        image = self.test_image3
        bdm_emptyvol_dev = '/dev/vdd'
        bdm_snapshot_dev = '/dev/vdc'
        bdm_ephemeral_dev = '/dev/vdb'
        self.status('Original Image block device map:')
        self.tester.print_block_device_map(image.block_device_mapping)
        bdm = self.add_block_device_types_to_mapping(device_name=bdm_ephemeral_dev,
                                                     no_device=True)
        self.add_block_device_types_to_mapping(device_name=bdm_snapshot_dev,
                                               no_device=True,
                                               block_device_map=bdm)
        self.add_block_device_types_to_mapping(device_name = bdm_emptyvol_dev,
                                               no_device=True,
                                               block_device_map=bdm)
        self.status('Applying the following block device map:')
        self.tester.print_block_device_map(bdm)
        instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
        self.status('Resulting in the instance block device map:')
        self.tester.print_block_device_map(instance.block_device_mapping)
        self.status('Checking instance, confirming lack of ephemeral disk...')
        try:
            instance.get_ephemeral_dev()
        except:
            self.debug('Ephemeral was not found, passing...')
        else:
            raise Exception('Ephemeral device found, but was over written with no_device?')
        guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                            md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                            map_device=instance.root_device_name)
        self.status('Checking for any non-root block devices...')
        remaining_devs = self.find_remaining_devices(instance=instance, known_dev_list=[guest_root_dev])
        if remaining_devs:
            raise Exception('Found additional unknown block devices on guest, expected all to be removed:' +
                            str(',').join(remaining_devs))
        self.status('Check block device mapping meta data...')
        instance.check_instance_meta_data_for_block_device_mapping()
        try:
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)

    def run_time_test4_image4_overwrite_misc_attributes_and_mixed_dot_at_run_time(self):
        errmsg = ""
        image = self.test_image4
        bdm_emptyvol_dev = '/dev/vdd'
        bdm_snapshot_dev = '/dev/vdc'
        bdm_ephemeral_dev = '/dev/vdb'
        bdm_emptyvol_size = image.block_device_mapping.get(bdm_emptyvol_dev).size + 1
        bdm_emptyvol_dot =  (not image.block_device_mapping.get(bdm_emptyvol_dev).delete_on_termination)
        bdm_snapshot_size = image.block_device_mapping.get(bdm_snapshot_dev).size + 1
        bdm_root_size = image.block_device_mapping.get(image.root_device_name).size + 1
        bdm_dot_value = (not image.block_device_mapping.get(image.root_device_name).delete_on_termination)
        self.status('Original Image block device map:')
        self.tester.print_block_device_map(image.block_device_mapping)
        bdm = self.add_block_device_types_to_mapping(device_name=image.root_device_name,
                                                           size=bdm_root_size,
                                                           delete_on_terminate=bdm_dot_value)
        self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                               device_name=bdm_snapshot_dev,
                                               delete_on_terminate=False,
                                               size = bdm_snapshot_size,
                                               block_device_map=bdm)
        self.add_block_device_types_to_mapping(device_name = bdm_emptyvol_dev,
                                               size = bdm_emptyvol_size,
                                               delete_on_terminate=bdm_emptyvol_dot,
                                               block_device_map=bdm)
        self.status('Applying the following block device map:')
        self.tester.print_block_device_map(bdm)
        instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance for ephemeral disk and size...')
            guest_ephemeral_dev = instance.check_ephemeral_against_vmtype()

            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also \
                         record the volume id, md5 and guest device within the instance for later stop, start, and detach \
                         operations...')
            self.status('Checking device(s) size(s) on guest vs requested in bdm...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            guest_root_size = instance.get_blockdev_size_in_bytes(guest_root_dev) / self.gig
            if guest_root_size != bdm_root_size:
                raise Exception('Root volume size on guest:'+ str(guest_root_size)+', != ' + str(bdm_root_size) +
                                ' the size requested for root in bdm')

            guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                                md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                                map_device=bdm_snapshot_dev)
            guest_snapvol_size = instance.get_blockdev_size_in_bytes(guest_snap_dev) / self.gig
            if guest_snapvol_size != bdm_snapshot_size:
                raise Exception('Volume size on guest:'+ str(guest_snapvol_size)+' != ' + str(bdm_snapshot_size) +
                                ', the size requested for bdm snap:' + str(self.base_test_snapshot.id))

            self.status("Attempting to find guest's device empty volume by process of elimination...")
            remaining_devs=self.find_remaining_devices(instance,[guest_root_dev, guest_snap_dev, guest_ephemeral_dev])
            if len(remaining_devs) != 1:
                raise Exception('Could not find empty vol dev from remaining devs on guest:' + str(",").join(remaining_devs))
            guest_emptyvol_device = '/dev/' + str(remaining_devs[0]).replace('/dev/','')
            self.status('Found empty volume on guest device:'+ str(guest_emptyvol_device)+', checking for proper size...')
            guest_emptyvol_size = instance.get_blockdev_size_in_bytes(guest_emptyvol_device) / self.gig
            if guest_emptyvol_size != bdm_emptyvol_size:
                raise Exception('Block device size on guest:' + str(guest_emptyvol_size) +
                                ", does not match requested size:" + str(bdm_emptyvol_size))
            self.status('Found empty volume device on guest, writing random  data and storing md5 prior to stop/start tests...')
            empty_vol = self.tester.get_volume(instance.block_device_mapping.get(bdm_emptyvol_dev).volume_id)
            empty_vol.guestdev = guest_emptyvol_device
            if not empty_vol in instance.attached_vols:
                instance.attached_vols.append(empty_vol)
            instance.vol_write_random_data_get_md5(empty_vol)
            self.status('Check block device mapping meta data...')
            #Temp work around for existing bug where ephemeral is not reported...
            meta_bdm = instance.block_device_mapping
            if not meta_bdm.has_key(bdm_ephemeral_dev):
                self.resulterr('Ephemeral disk not reported in instance block dev mapping: see euca-6048')
                eph_dev = BlockDeviceType()
                eph_dev.device_name=bdm_ephemeral_dev
                eph_dev.ephemeral_name='ephemeral0'
                meta_bdm[bdm_ephemeral_dev] = eph_dev
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=meta_bdm)

            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)


    def run_time_test5_image1_add_snap_map_attach_a_vol_to_running_instance(self):
        '''
        Will test running a default bfebs image registered w/ the following mapped devices, delete on terminate for
        all devices is False:
        1)the snapshot containing the original bfebs image

        At run time the following block devices will be applied per map:
        1) Single volume created from the local base_test_snapshot - size altered by +1 gig

        Post running:
        1) Attach the base_test_volume

        Additional tests here:
        -will verify attach, detach, reattach of single volume to instance in running state
        -will verify stop, start, terminate, etc of root vol, run time bdm vol, and volume attached during running state
        -will verify a portion of the root device's md5sum against the original volume's/snapshot's
        -will verify a portion of the test snapshot's md5sum against the guest's device and size of disk
        -will attempt to verify volume states through stopping and restarting the instance
        -will attempt to verify instance meta data for the block device map in use
        -will verify delete on terminate set to True, and all volumes are deleted post instance termination
        '''
        errmsg = ""
        image = self.test_image1
        bdm_emptyvol_dev = '/dev/vdd'
        bdm_snapshot_dev = '/dev/vdc'
        bdm_ephemeral_dev = '/dev/vdb'
        bdm_ephemeral_name = 'ephemeral0'
        bdm_root_size = image.block_device_mapping.get(image.root_device_name).size
        bdm_emptyvol_size = 1
        bdm_snap_size = self.base_test_snapshot.volume_size + 1


        self.status('Using image:' +str(self.test_image1.id)+ ", ephemeral, and snapshot ebs devices added at run time..")
        self.status('Original Image block device map:')
        self.tester.print_block_device_map(image.block_device_mapping)
        bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                                     device_name=bdm_snapshot_dev,
                                                     size=bdm_snap_size,
                                                     delete_on_terminate=True)

        #add ephemeral dev separately to use later for bug work around...
        eph_dev = BlockDeviceType()
        eph_dev.device_name=bdm_ephemeral_dev
        eph_dev.ephemeral_name=bdm_ephemeral_name
        bdm[bdm_ephemeral_dev] = eph_dev
        self.status('Applying the following block device map:')
        self.tester.print_block_device_map(bdm)
        instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
        try:
            self.current_test_instance = instance
            self.status('Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance for to make sure ephemeral is present...')
            instance.get_ephemeral_dev()

            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also \
                         record the volume id, md5 and guest device within the instance for later stop, start, and detach \
                         operations...')
            self.status('Getting guest dev for bdm root device...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            self.status('Checking guest root device size against bdm')
            guest_root_size = instance.get_blockdev_size_in_bytes(guest_root_dev) / self.gig
            if guest_root_size != bdm_root_size:
                raise Exception('Root volume size on guest:'+ str(guest_root_size)+', != ' + str(bdm_root_size) +
                                ' the size requested for root in bdm')
            self.status('Getting guest device for bdm base test snapshot...')
            guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                                md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                                map_device=bdm_snapshot_dev)
            self.status('Checking size on guest for base test snapshot...')
            guest_snapvol_size = instance.get_blockdev_size_in_bytes(guest_snap_dev) / self.gig
            if guest_snapvol_size != bdm_snap_size:
                raise Exception('Volume size on guest:'+ str(guest_snapvol_size)+' != ' + str(bdm_snap_size) +
                                ', the size requested for bdm snap:' + str(self.base_test_snapshot.id))
            #Grab the current block device map before attaching any volumes, use this for meta data testing later.
            #Temp work around for existing bug where ephemeral is not reported...
            meta_bdm = instance.block_device_mapping
            if not meta_bdm.has_key(bdm_ephemeral_dev):
                self.resulterr('Ephemeral disk not reported in instance block dev mapping: see euca-6048')
                meta_bdm[bdm_ephemeral_dev] = eph_dev
            new_vol = self.tester.create_volume(zone=self.zone,size=1, timepergig=180)
            self.status('Attaching new test volume to this running instance...')
            instance.attach_euvolume(new_vol, timeout=120, overwrite=False)
            self.status('Block dev map after attaching volume to running instance:')
            instance.update()
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Detaching the recently attached volume from instance...')
            instance.detach_euvolume(self.base_test_volume)
            self.status('Block dev map after detaching volume from instance...')
            instance.update()
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Attaching the same test volume to this running instance for remainder of test...')
            instance.attach_euvolume(new_vol, timeout=120, overwrite=True)
            self.status('Block dev map after re-attaching volume to running instance:')
            instance.update()
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Check block device mapping meta data...')
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=meta_bdm)
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)


    def misc_test1_exceed_max_vol_size_storage_property_per_block_dev_map(self):
        '''
        Attempts to set the maxvolumesizeingb storage property to a value that will be exceeded by the size of the ebs
        volume(s) requested in the image block device mapping, as well as mapping provided at run time. This test
        will fail if this operation is permitted.
        '''
        errmsg = ""
        bdm_snapshot_dev = '/dev/vdc'
        bdm_snap_size = 6
        orig_maxsize = self.tester.property_manager.get_storage_maxvolumesizeingb_value()
        try:
            self.tester.property_manager.set_storage_maxvolumesizeingb_value(1)
            image = self.test_image1
            self.status('Using image:' +str(self.test_image1.id)+ ", ephemeral, and snapshot ebs devices added at run time..")
            self.status('Original Image block device map:')
            self.tester.print_block_device_map(image.block_device_mapping)
            bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                                         device_name=bdm_snapshot_dev,
                                                         size=bdm_snap_size,
                                                         delete_on_terminate=True)
            self.status('Applying the following block device map:')
            self.tester.print_block_device_map(bdm)
            try:
                instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
                self.current_test_instance = instance
            except:
                self.status('Instance failed, did not exceed properties correctly. Passing')
            else:
                try:
                    instance.terminate_and_verify()
                except:pass
                raise Exception('Instance did not fail, storage property "maxvolumesizeingb" may have been exceeded')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            self.tester.property_manager.set_storage_maxvolumesizeingb_value(orig_maxsize)
            if errmsg:
                raise Exception(errmsg)

    def misc_test2_exceed_max_total_storage_property_per_block_dev_map(self):
        '''
        Attempts to set maxvolumesizeingb storage property to a value that is than exceeded per the ebs volumes
        requested in the block device mapping of the image and and run time. Test will fail if this operation is
        permitted
        '''
        errmsg = ""
        bdm_snapshot_dev = '/dev/vdc'
        bdm_snap_size = 6
        orig_maxtotal = self.tester.property_manager.get_storage_maxtotalvolumesizeingb_value()
        try:
            self.tester.property_manager.set_storage_maxtotalvolumesizeingb_value(5)
            image = self.test_image1
            self.status('Using image:' +str(self.test_image1.id)+ ", ephemeral, and snapshot ebs devices added at run time..")
            self.status('Original Image block device map:')
            self.tester.print_block_device_map(image.block_device_mapping)
            bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                                         device_name=bdm_snapshot_dev,
                                                         size=bdm_snap_size,
                                                         delete_on_terminate=True)
            self.status('Applying the following block device map:')
            self.tester.print_block_device_map(bdm)
            try:
                instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
                self.current_test_instance = instance
            except:
                self.status('Instance failed, did not exceed properties correctly. Passing')
            else:
                try:
                    instance.terminate_and_verify()
                except:pass
                raise Exception('Instance did not fail, storage property "maxtotalvolumesizeingb" may have been exceeded')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            self.tester.property_manager.set_storage_maxtotalvolumesizeingb_value(orig_maxtotal)
            if errmsg:
                raise Exception(errmsg)

    def misc_test3_run_image1_check_attached_volume_states_during_stop_start(self, can_detach_block_dev_map=False):
        '''
        Attempts to run test bfebs image#1 and test the following:
        -Add an additional ebs block device in the map built from the base test snapshot
        -Verify the root device is present and in bdm and by checksum on the guest
        -Verify the root device's size specified in the bdm is also the size which appears on the running guest
        -Verify the additional ebs device is present and in bdm and by checksum on the guest
        -Verify the additional ebs device's size specified in the bdm is also the size which appears on the running guest
        -Verify instance meta data for block device mapping
        -Attach a volume while instance is in running state, verify device on guest
        -Stop the instance and verify the instance goes to stopped state correctly
        -While in the stopped state monitor all volumes attached to instance for 60 seconds to confirm correct state
        -Monitor instance for 60 seconds to verify it remains in correct stopped state
        -Attempt to detach volume while instance is in stopped state
        -Start instance and verify all volumes are attached, check guest for devices by checksum


        '''
        errmsg = ""
        image = self.test_image1
        bdm_snapshot_dev = '/dev/vdc'
        bdm_snap_size = self.base_test_snapshot.volume_size
        bdm_root_size = image.block_device_mapping.get(image.root_device_name).size
        try:
            self.status('Original Image block device map:')
            self.tester.print_block_device_map(image.block_device_mapping)
            bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                                         device_name=bdm_snapshot_dev,
                                                         size=bdm_snap_size,
                                                         delete_on_terminate=True)
            self.status('Applying the following block device map:')
            self.tester.print_block_device_map(bdm)
            self.status('Running instance with previously displayed block device mapping...')
            instance = self.tester.run_image(image=image,
                                             block_device_map=bdm,
                                             keypair=self.keypair,
                                             password=self.instance_password,
                                             group=self.group)[0]

            self.current_test_instance = instance
            self.status('Instance now running. Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also \
                         record the volume id, md5 and guest device within the instance for later stop, start, and detach \
                         operations...')
            self.status('Getting guest device for block dev map root device...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            self.status('Checking device(s) size(s) on guest vs requested in bdm for root device ...')
            root_dev_size = instance.get_blockdev_size_in_bytes(guest_root_dev) / self.gig
            if root_dev_size != bdm_root_size:
                raise Exception('Root device size on guest:' + str(root_dev_size) +
                                ' !=  requested size' + str(bdm_root_size) +
                                ", original size in image:" + str(bdm_root_size))
            self.status('Getting guest device for block dev map, device using base test snapshot...')
            guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                                md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                                map_device=bdm_snapshot_dev)
            self.status('Checking device(s) size(s) on guest vs requested in bdm for base test snapshot ...')
            guest_snapvol_size = instance.get_blockdev_size_in_bytes(guest_snap_dev) / self.gig
            if guest_snapvol_size != bdm_snap_size:
                raise Exception('Volume size on guest:'+ str(guest_snapvol_size)+' != ' + str(bdm_snap_size) +
                                ', the size requested for bdm snap:' + str(self.base_test_snapshot.id))
            #Get block device map before attaching volumes to use for meta data test later...
            meta_bdm = instance.block_device_mapping
            self.status('Attaching the base test volume to this running instance...')
            instance.attach_euvolume(self.base_test_volume, timeout=120, overwrite=False)
            self.status('Block dev map after attaching volume to running instance:')
            instance.update()
            self.status('Check block device mapping meta data...')
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=meta_bdm)
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Monitor attached volumes for 1 minute make sure status remains in-use...')
            start = time.time()
            elapsed = 0
            errmsg = ''
            while elapsed < 60:
                elapsed = int(time.time()- start)
                for vol in instance.attached_vols:
                    vol.update()
                    if vol.status != 'in-use':
                        err = str(vol.id)+", state is not in-use while instance:" + str(instance.id) \
                                  + ' is stopped, elapsed:' +str(elapsed) + "\n"
                        self.debug(err)
                        errmsg += err
                    if instance.state != 'stopped':
                        err  = str(instance.id) + " instance state: " + str(instance.state) \
                                  + " no longer in stopped state, elapsed:" + str(elapsed) + "\n"
                        self.debug(err)
                        errmsg += err
                    self.tester.print_euvolume_list(instance.attached_vols)
                time.sleep(10)
            self.status('Attempt to detach volume which was attached after running instance while in stopped state...')
            instance.detach_euvolume(self.base_test_volume,timeout=180)
            try:
                self.status('Attempt to detach volume from block dev map...')

            except Exception, e:
                err = ('Could not detach bdm volume:' + str(self.base_test_volume.id)
                       + " from stopped instance:" + str(instance.id) + ", err:"+ str(e)) + '\n'
                if can_detach_block_dev_map:
                    errmsg += err
                else:
                    self.debug('WARNING' + err + "\n" +
                               'can_detach_block_dev_map not set, may not be supported in this release')

            self.status('Restarting instance:' + str(instance.id))
            instance.start_instance_and_verify(checkvolstatus=True)
            self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' +str(e)
            self.endfailure(errmsg)

        finally:
            instance.terminate_and_verify()
            if errmsg:
                raise Exception(errmsg)

    def misc_test4_run_image1_terminate_during_stopped_verify_volume_dot(self):
        '''
        Attempts to run an instance with it's:
        - root device DOT (delete on termination) flag set to True
        - add another volume in the BDM with the DOT flag set to False
        - verifies that the 2nd non-root volume is present on the guest
        - stops the guest
        - terminates the guest in the stop state
        - verifies the volume states match the DOT states. The root volume should be deleted, the 2nd should be available
        '''
        self.status('Running test image1 w/ dot flag set to true and BDM volume with DOT set to false...')
        image = self.test_image1
        bdm_snapshot_dev = '/dev/vdc'
        self.status('Original Image block device map:')
        self.tester.print_block_device_map(image.block_device_mapping)
        bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                                     device_name=bdm_snapshot_dev,
                                                     delete_on_terminate=False)
        self.status('Applying the following block device map:')
        self.tester.print_block_device_map(bdm)
        self.status('Running instance with previously displayed block device mapping...')
        instance = self.tester.run_image(image=image,block_device_map=bdm, keypair=self.keypair, group=self.group)[0]
        self.current_test_instance = instance
        self.status('Checking instance for attached BDM volume created from base test snapshot')
        guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                            md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                            map_device=bdm_snapshot_dev)
        self.status('Stopping instance...')
        instance.stop_instance_and_verify()
        self.status('Terminating instance and verifying correct volume states for both BDM volumes root + ' +
                    str(bdm_snapshot_dev))
        instance.terminate_and_verify()

    def misc_test5_run_image1_with_multiple_bdm_terminate_in_stopped_state(self, can_detach_block_dev_map=False):
        '''
        Attempts to run test bfebs image#1 and test the following:
        -Add an additional ebs block device in the map built from the base test snapshot
        -Verify the root device is present and in bdm and by checksum on the guest
        -Verify the root device's size specified in the bdm is also the size which appears on the running guest
        -Verify the additional ebs device is present and in bdm and by checksum on the guest
        -Verify the additional ebs device's size specified in the bdm is also the size which appears on the running guest
        -Verify instance meta data for block device mapping
        -Attach a volume while instance is in running state, verify device on guest
        -Stop the instance and verify the instance goes to stopped state correctly
        -While in the stopped terminate the instance
        -Verify all volumes enter their correct Delete On Terminate states

        '''
        errmsg = ""
        image = self.test_image1
        bdm_snapshot_dev = '/dev/vdc'
        bdm_snap_size = self.base_test_snapshot.volume_size
        bdm_root_size = image.block_device_mapping.get(image.root_device_name).size
        try:
            self.status('Original Image block device map:')
            self.tester.print_block_device_map(image.block_device_mapping)
            bdm = self.add_block_device_types_to_mapping(snapshot_id=self.base_test_snapshot.id,
                                                         device_name=bdm_snapshot_dev,
                                                         size=bdm_snap_size,
                                                         delete_on_terminate=True)
            self.status('Applying the following block device map:')
            self.tester.print_block_device_map(bdm)
            self.status('Running instance with previously displayed block device mapping...')
            instance = self.tester.run_image(image=image,
                                             block_device_map=bdm,
                                             keypair=self.keypair,
                                             password=self.instance_password,
                                             group=self.group)[0]

            self.current_test_instance = instance
            self.status('Instance now running. Resulting in the instance block device map:')
            self.tester.print_block_device_map(instance.block_device_mapping)
            self.status('Checking instance devices for md5sums which match original volume/snapshots.\nThis step will also \
                         record the volume id, md5 and guest device within the instance for later stop, start, and detach \
                         operations...')
            self.status('Getting guest device for block dev map root device...')
            guest_root_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.build_image_snapshot.eutest_volume_md5,
                                                                                md5len=self.build_image_snapshot.eutest_volume_md5len,
                                                                                map_device=instance.root_device_name)
            self.status('Checking device(s) size(s) on guest vs requested in bdm for root device ...')
            root_dev_size = instance.get_blockdev_size_in_bytes(guest_root_dev) / self.gig
            if root_dev_size != bdm_root_size:
                raise Exception('Root device size on guest:' + str(root_dev_size) +
                                ' !=  requested size' + str(bdm_root_size) +
                                ", original size in image:" + str(bdm_root_size))
            self.status('Getting guest device for block dev map, device using base test snapshot...')
            guest_snap_dev = instance.get_guest_dev_for_block_device_map_device(md5=self.base_test_snapshot.eutest_volume_md5,
                                                                                md5len=self.base_test_snapshot.eutest_volume_md5len,
                                                                                map_device=bdm_snapshot_dev)
            self.status('Checking device(s) size(s) on guest vs requested in bdm for base test snapshot ...')
            guest_snapvol_size = instance.get_blockdev_size_in_bytes(guest_snap_dev) / self.gig
            if guest_snapvol_size != bdm_snap_size:
                raise Exception('Volume size on guest:'+ str(guest_snapvol_size)+' != ' + str(bdm_snap_size) +
                                ', the size requested for bdm snap:' + str(self.base_test_snapshot.id))
                #Get block device map before attaching volumes to use for meta data test later...
            meta_bdm = instance.block_device_mapping
            self.status('Attaching the base test volume to this running instance...')
            instance.attach_euvolume(self.base_test_volume, timeout=120, overwrite=False)
            self.status('Block dev map after attaching volume to running instance:')
            instance.update()
            self.status('Check block device mapping meta data...')
            instance.check_instance_meta_data_for_block_device_mapping(root_dev=image.root_device_name, bdm=meta_bdm)
            self.status('Stopping instance:' + str(instance.id))
            instance.stop_instance_and_verify()
            self.status('Monitor attached volumes for 1 minute make sure status remains in-use...')
            start = time.time()
            elapsed = 0
            errmsg = ''
        except Exception, e:
            tb = self.tester.get_traceback()
            errmsg = str(tb) + '\nTest Failed, err:' + str(e)
            self.endfailure(errmsg)

        finally:
            try:
                self.status('Terminating instance, checking for delete on termination status for ebs block devs...')
                instance.terminate_and_verify()
            except Exception, tv:
                tb = self.tester.get_traceback()
                errmsg = errmsg + "\n" + str(tb) + "\nError during terminate and verify:" + str(tv)
            if errmsg:
                raise Exception(errmsg)



    def find_remaining_devices(self,instance,known_dev_list):
        self.debug('Looking for remaining devices after known devs:' +str(',').join(known_dev_list))
        dev_dir = instance.get_dev_dir()
        remove_devs = []
        for dev in dev_dir:
            dev_string = '/dev/' + str(dev).replace('/dev/','')
            for known_dev in known_dev_list:
                #Remove all disk partitions
                if dev_string.startswith(known_dev):
                    self.debug(str(dev)+': Eliminating known dev:' +str(dev_string)+ ' from possible remaining devices')
                    if not dev in remove_devs:
                        remove_devs.append(dev)
                    continue
        for dev in remove_devs:
            self.debug('Removing dev:'+str(dev))
            dev_dir.remove(dev)
        self.debug('Returning list of possible remaining/unknown devices:' + str(',').join(dev_dir))
        return dev_dir



if __name__ == "__main__":
    testcase = Block_Device_Mapping_Tests()

    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list

    if testcase.args.tests:
        list = testcase.args.tests.splitlines(',')
    else:
        if testcase.args.use_previous:
            list = ['try_existing_resources_else_create_them']
        else:
            if not testcase.args.url:
                print 'URL needed if not using previously populated test resources and "use_previous" flag'
                testcase.parser.print_help()
                exit(1)
            list = ['setup_bfebs_instance_volume_and_snapshots_from_url']
        list.extend(['register_n_run_test1_bfebs_image_w_dot_is_true',
                     'register_n_run_test2_bfebs_image_w_dot_is_false',
                     'register_n_run_test3_bfebs_image_w_ephemeral_map_snap_map_and_ebsvol_map_dot_is_true',
                     'register_n_run_test4_bfebs_image_w_ephemeral_map_snap_map_and_ebsvol_map_dot_is_false',
                     'run_time_test1_image1_overwrite_root_vol_size_and_set_dot_to_false',
                     'run_time_test2_image1_add_ephemeral_map_snap_map_emptyvol_map_at_run_time_dot_true',
                     'run_time_test3_image3_overwrite_all_non_root_w_no_device',
                     'run_time_test4_image4_overwrite_misc_attributes_and_mixed_dot_at_run_time',
                     'run_time_test5_image1_add_snap_map_attach_a_vol_to_running_instance',
                     'misc_test1_exceed_max_vol_size_storage_property_per_block_dev_map',
                     'misc_test2_exceed_max_total_storage_property_per_block_dev_map',
                     'misc_test3_run_image1_check_attached_volume_states_during_stop_start',
                     'misc_test4_run_image1_terminate_during_stopped_verify_volume_dot',
                     'misc_test5_run_image1_with_multiple_bdm_terminate_in_stopped_state'])

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,eof=False,clean_on_exit=True)
    exit(result)



#!/usr/bin/env python

#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#
# [test_ebs_basic_test_suite]
#       Full suite of ebs related tests
#        Test Summary: 
#
#        -create a volume (do this first)
#        -run an instance (do this second, if this fails at least we know we could create a vol)
#        
#        Usage Tests: 
#        -negative -attempt to attach a volume to an instance in a separate cluster. 
#        -attach a single volume to an instance in the zones given, write random data and calc md5 of volumes
#        -negative:attempt to delete the attached instance, should fail
#        -negative:attempt to attach an in-use volume, should fail
#        -attach a 2nd volume to an instance, write random date to vol and calc md5 of volumes
#        -reboot instance
#        -verify both volumes are attached after reboot of instance. 
#        -detach 1st volume
#        -create snapshot of detached volume
#        -create snapshot of attached volume
#        -attempt to create a volume of each snapshot, if within a multi-cluster env do 1 in each cluster 
#        -attempt to attach each volume created from the previous snaps to an instance verify md5s
#        -'if' a bfebs instance was used, attempt to stop and detach volumes while in stopped state
#        -terminate all instances used in this test, verify any attached volumes return to available state
#        
#        Properties tests:
#        -create a volume of greater than prop size, should fail
#        -create a 2nd volume attempting to exceed the max aggregate size, should fail
#        
#        
#        Cleanup:
#        --remove all volumes, instance, and snapshots created during this test
#
#    @author: clarkmatthew


from eutester.eutestcase import EutesterTestCase
from ebstestsuite import EbsTestSuite


ebssuite = None
zone = None
config_file = None
password = None
credpath = None
keypair = None
group = None
vmtype = None
emi = None


if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch

    testcase= EutesterTestCase(name='ebs_basic_test')    
    testcase.setup_parser(description="Attempts to test and provide info on focused areas related to Eucalyptus EBS related functionality.", 
                          testlist=False)
    testcase.parser.add_argument('--instance_password',dest='inst_pass', help='String representing root password for instance, will replace keypair ssh login',default=None)
    testcase.parser.add_argument('--reboot_timeout',dest='waitconnect', help='Time to wait before trying to connect to rebooted guests',default=30)

    testcase.get_args()
    ebstestsuite= testcase.do_with_args(EbsTestSuite)
    testcase.clean_method = ebstestsuite.clean_created_resources
    testlist = ebstestsuite.ebs_basic_test_suite(run=False)
    ret = testcase.run_test_case_list(testlist)
    print "ebs_basic_test exiting:("+str(ret)+")"
    exit(ret)

    
  



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#


'''
Created on Apr 13, 2012

@author: clarkmatthew

#note:
currently the test wrappers assume eucarc in the users home dir 

example:
    from eustoretests import Eustoretestsuite
    #create the eustore test suite...
    et = Eustoretestsuite(config_file="2b_tested.lst", password="foobar")
    
    #get a list of available eustore images which are not installed on the local system...
    list = et.get_uninstalled_summary()
    
    #attempt to install images from the list we just created
    #this is redundant as this will skip images found to already be installed by default
    et.install_all_images(list=list)
    
    #Attempt to run an instance and perform test suite against a list of images,
    #by default will attempt to run against all images in et.list
    et.test_image_list()
    
    #The results are redirected to logs created per image, but can be dumped to stdout using...
    #et.print_image_list_results()
    #This will print results and info for all images in et.list
    #...or print for 1 image by something like the following...
    image = et.get_eustore_image_by_string('3235725435')
    image.printdata()
    
    #this should produce something like the following...
    ------------------3235725435_CentOS_5_x86_64----------------------
    kerneltype = Single
    ver = CentOS
    name = 3235725435_CentOS_5_x86_64
    eki = eki-EB153854
    remotestring = 3235725435 centos      x86_64  2012.1.14      CentOS 5 1.3GB root, Single Kernel20120114150503      28fc-4826   centos-based           images@lists.eucalyptus.com
    id2 = 20120114150503
    rev = 5
    id3 = 28fc-4826
    email = images@lists.eucalyptus.com
    emi = emi-2B3435D5
    eri = eri-CC9432DB
    user = root,
    type = centos-based
    date = 2012.1.14
    arch = x86_64
    id = 3235725435
    distro = centos
    TEST:running_test         RESULT:SUCCESS
    TEST:terminate_test       RESULT:SUCCESS
    TEST:reboot_test          RESULT:SUCCESS
    TEST:user_test            RESULT:SUCCESS
    TEST:volume_test          RESULT:SUCCESS
    TEST:install_test         RESULT:SUCCESS
    TEST:root_test            RESULT:SUCCESS
    TEST:metadata_test        RESULT:SUCCESS
    
    



      
'''

from eucaops import Eucaops
from eutester import euinstance, euvolume
import logging
from boto.ec2.snapshot import Snapshot
from boto.ec2.image import Image
import re
import time
import unittest
import inspect
import gc
import sys
import os
#import httplib


class User():
    name = ""
    uid = ""
    group= ""
    udir = ""
    
    def __init__(self,name="",uid="",group="",udir=""):
        self.name = name
        self.id = uid
        self.group = group
        self.dir = udir
        self.__name__ = name
        self.__str__ = name
        

class TestStatus():
    not_run = "not run"
    success = "SUCCESS"
    failed = "FAILED"
    
class ImageIndex():
    '''
                id          distro      arch      date      ver      rev    size    user       kerneltype            ----           id2            id3            type          email    
           ['4150406313', 'centos', 'i386', '2011.07.02', 'CentOS', '5', '1.3GB', 'root,', 'Hypervisor-Specific', 'Kernels', '20110716161844', '6e69-7565', 'centos-based', 'images@lists.eucalyptus.com']
    '''
    id = 0   
    distro =1
    arch = 2
    date = 3
    ver = 4
    rev = 5
    size = 6
    user = 7 
    kerneltype = 8 
    #junk = 9
    id2 = 10
    id3 = 11
    type = 12
    email = 13
    
class EustoreTests():
    install_test = "install_test"
    running_test = "running_test"
    metadata_test = "metadata_test"
    user_test = "user_test"
    root_test = "root_test"
    attach_volume_test = "attach_volume_test"
    detach_volume_test = "detach_volume_test"
    reboot_test = "reboot_test"
    terminate_test = "terminate_test"
    ssh_test = "ssh_test"
    zeroconf_test = "zeroconf_test"
    virtiopresent_test = "virtiopresent_test"

class EustoreImage():
    '''
    simple class to hold data and state related to eustore images and catalog information
    '''
    
    def __init__(self, 
                id = "",
                distro = "",
                arch = "",
                date = "",
                ver = "",
                rev = "",
                size = 0,
                user = "",
                kerneltype = "", 
                id2 = "",
                id3 = "",
                type = "",
                email = "",
                emi = '', 
                eki = '',
                eri = '',
                remotestring = '',
                loginuser = 'root',
                logdir='../artifacts/'):
        
        self.id = id   
        self.distro = distro
        self.arch = arch
        self.date = date
        self.ver = ver
        self.rev = rev
        self.size = float(size)
        self.user = user
        self.kerneltype = kerneltype 
        self.id2 = id2
        self.id3 = id3
        self.type = type
        self.email = email
        self.emi = emi
        self.eki = eki
        self.eri = eri
        self.remotestring = remotestring
        self.loginuser = loginuser
        self.reservation = []
        self.name = id+"_"+ver+"_"+rev+"_"+arch  
        if not os.path.exists(logdir):
            os.makedirs(logdir)
        self.logger = logging.getLogger(self.name)
        hdlr = logging.FileHandler(logdir+self.name+".log")
        formatter = logging.Formatter('%(asctime)s:[%(funcName)s()]:[%(lineno)d]:['+str(self.name)+']: %(message)s')
        hdlr.setFormatter(formatter)
        self.logger.addHandler(hdlr)
        self.logger.setLevel(logging.DEBUG)
        #self.debug = self.logger.debug
        self.results = { 
                EustoreTests.install_test : TestStatus.not_run,
                EustoreTests.ssh_test : TestStatus.not_run,
                EustoreTests.running_test : TestStatus.not_run,
                EustoreTests.metadata_test: TestStatus.not_run,
                EustoreTests.user_test : TestStatus.not_run,
                EustoreTests.zeroconf_test : TestStatus.not_run,
                EustoreTests.root_test : TestStatus.not_run,
                EustoreTests.virtiopresent_test : TestStatus.not_run,
                EustoreTests.attach_volume_test :TestStatus.not_run,
                EustoreTests.reboot_test : TestStatus.not_run,
                EustoreTests.terminate_test :  TestStatus.not_run
               }
        self.debug("######################START#############################", verbose = False)
        
    def __str__(self):
        return self.name
 
    def debug(self, msg, verbose=True):
        if verbose:
            print msg
        self.logger.debug(msg)
    
    def printdata(self,printmethod=None):
        if printmethod is None:
            printmethod = self.debug
        printmethod("\n------------------"+str(self.name)+"----------------------")
        for key in self.__dict__:
            value = self.__dict__[key]
            if ( isinstance(value, basestring) or isinstance(value,int) ):
                printmethod( str(key)+ " = " + str(value))
        self.printresults(printmethod=printmethod)
        
    def printresults(self,printmethod = None):
        if printmethod is None:
            printmethod = self.debug
        for test in self.results:
            printmethod( str("TEST:"+str(test)).ljust(25)+str(" RESULT:"+self.results[test]).ljust(0))
            
    def clearresults(self, exclude = [EustoreTests.install_test] ):
        for test in self.results:
            for ex in exclude:
                if test == ex:
                    continue
            self.results[test]=TestStatus.not_run
            
        

class Eustoretestsuite():
    cur_image = None
    
    def __init__(self, tester=None,  config_file='../input/2b_tested.lst', password="foobar", credpath=None, url=None, list=None, volumes=None, keypair=None, group=None, image=None, zone='PARTI00',  eof=1):
        if tester is None:
            self.tester = Eucaops( config_file=config_file, password = password, credpath=credpath)
            self.tester.exit_on_fail = eof
        else:
            self.tester = tester
            self.tester.exit_on_fail = eof
        self.zone = zone
        self.url = url
        self.url_export_string = str(" ")
        if self.url is not None:
            self.url_export_string = "export EUSTORE_URL="+str(self.url)+" &> /dev/null &&"
        if list is None:
            self.list = self.get_eustore_images()
        else:
            self.list = list
            
        self.sync_installed_list()
        self.cur_image = None
        
        #Test specific resources 
        self.group = self.get_a_group()
        self.keypair = self.get_a_key()
        self.test_volumes = []
        self.test_snapshots = []
        
        
            
    def get_a_key(self):    
        '''
        helper function to get a key to use for the instance related testing
        '''
        keypair = None
        keys = self.tester.get_all_current_local_keys()
        if keys != []:
            keypair = keys[0]
        if keypair is None:
            try:
                keypair = self.tester.add_keypair("eustoretest" + "-" + str(time.time()))
            except Exception, e:
                raise Exception("Error when adding keypair. Error:"+str(e))
        return keypair
            
    def get_a_group(self, group_name="eustoregroup"):
        '''
        helper function to fetch a group to use for the instance related testing
        '''
        try:
            group = self.tester.add_group(group_name)
            self.tester.authorize_group_by_name(group.name)
            self.tester.authorize_group_by_name(group.name,protocol="icmp",port=-1)
        except Exception, e:    
            raise Exception("Error when setting up group:"+str(group_name)+", Error:"+str(e))
        return group
              
    def debug(self,msg):
        if self.cur_image is not None:
            self.cur_image.debug(msg)
        return self.tester.debug(msg)
    
    
    def create_image_from_emi(self,emi,name=None, id=None, size=2, add=True):
        '''
        Method used to build a eustore image object from an existing EMI. Mainly used to create an object that 
        can be fed into the test suite. By default this will append the image to this Eustoretestsuite image list
        Returns a eustore image object
        emi - mandatory - emi image object
        name -optional- name used in eustore object to be created
        id -optional- id used in eustore object to be created
        size -optional- size used in eustore object to be created
        add -optional- boolean- add to local eustore list of this eustoretestsuite
        '''
        newimg = EustoreImage()
        if id is None:
            newimg.id = emi.id
        else:
            newimg.id = id
            
        if name is None:
            newimg.name = emi.location
        else:
            newimg.name = name
            
        newimg.eki = emi.kernel_id
        newimg.eri = emi.ramdisk_id
        newimg.emi = emi.id
        newimg.size = size
        if add:
            self.list.append(newimg)
        return newimg

        
    def convert_image_list_to_eustore_images(self,list, prefix="", size=2, add=True):
        '''
        Method to convert list of non-eustore system images to eustore images. Returns list of the converted eustore images
        list - mandatory - list of images to convert
        prefix - a string prefixed to the stored image name
        id - a string to be used as the image's id, otherwise will use the underlying image's location string
        size - expected size of the image, used for determining the appropriate minimum instance type when running
        add - boolean, used to determine if this eustore image should be added to the main eustore image list
        '''
        retlist = []
        for image in list:
            name = str(prefix)+"-"+str(image.location)
            retlist.append(self.create_image_from_emi(image, name=name, id=image.location, size=size, add=add))
        return retlist
        
        
    
    def get_eustore_images(self):
        '''
        Attempts to populate a list of images by retrieving image attributes from eustore command(s).   
        '''
        ii = ImageIndex()
        images = []
        out = self.tester.sys( self.url_export_string+" eustore-describe-images -v")
        l = 0
        while (l < len(out)):
            line = out[l].strip().split()+out[l+1].strip().split()
            image = EustoreImage(
                                id = line[ii.id],
                                distro = line[ii.distro],
                                arch = line[ii.arch],
                                date = line[ii.date],
                                ver = line[ii.ver],
                                rev = line[ii.rev],
                                size = float(re.sub("GB","", line[ii.size])),
                                user = line[ii.user],
                                kerneltype = line[ii.kerneltype], 
                                id2 = line[ii.id2],
                                id3 = line[ii.id3],
                                type = line[ii.type],
                                email = line[ii.email],
                                remotestring = out[l].strip()+out[l+1].strip(),
                                )
            image.loginuser = self.get_default_login_user_by_distro(ii.distro)
            
            images.append(image)
            l += 2
        return images
            
    def get_default_login_user_by_distro(self,distro):
        '''
        helper method to return default login user per distro
        '''
        if re.search('UBUNTU', str(distro).upper()):
            return 'ubuntu'
        else:
            return 'root'
            
    def get_system_images(self, emi=None, root_device_type=None, root_device_name=None, location=None, state="available", arch=None, owner_id=None):
        """
        Get images matching the provided criteria. 
        By default matches image location containing the  string held by the 'location' option.
        emi              (optional string ) Partial ID of the emi to return, ie to grab all EMI use emi="emi-"prefix to grab any
        root_device_type (optional string)  example: 'instance-store' or 'ebs'
        root_device_name (optional string)  example: '/dev/sdb' 
        location         (optional string)  partial on location match example: 'centos'
        state            (optional string)  example: 'available'
        arch             (optional string)  example: 'x86_64'
        owner_id         (optional string) owners numeric id
        """
        retlist = []    
        images = self.tester.ec2.get_all_images()
        for image in images:
            
            if (emi is not None) and (not re.search(emi, image.id)):      
                continue  
            if ((root_device_type is not None) and (image.root_device_type != root_device_type)):
                continue            
            if ((root_device_name is not None) and (image.root_device_name != root_device_name)):
                continue       
            if ((state is not None) and (image.state != state)):
                continue            
            if ((location is not None) and (not re.search( location, image.location))):
                continue           
            if ((arch is not None) and (image.architecture != arch)):
                continue                
            if ((owner_id is not None) and (image.owner_id != owner_id)):
                continue
            
            retlist.append(image)
        return retlist
            
            
            
    def get_registered_emis(self, eustore_image,  emi="emi-"):
        '''
        Attempts to return a list of emis using this eustore image
        eustore_image - mandatory - eustore image object
        emi - optional - string used to filter matched images 
        '''
        images = self.get_system_images(emi, location=eustore_image.id)    
        if images == []:
            return None
        else:
            return images
    
    def get_registered_ekis(self, eustore_image):
        '''
        Attempts to get a list of ekis using this eustore image
        eustore_image - mandatory - eustore image object
        '''
        return self.get_registered_emis(eustore_image, emi="eki-")
    
    def get_registered_eris(self, eustore_image):
        '''
        Attempts to get a list of eris using this eustore image
        eustore_image - mandatory - eustore image object
        '''
        return self.get_registered_emis(eustore_image, emi="eri-")
    
        
   
            
    def install_image(self,eustore_image,
                      help=False,
                      tarball=None,
                      description=None,
                      arch=None,
                      prefix=None,
                      bucket=None,
                      kernel_type=None,
                      dir="/disk1/storage",
                      kernel=None,
                      ramdisk=None,
                      url=None,
                      region=None,
                      access_key=None,
                      secret_key=None,
                      force=False, 
                      eucarc=False,
                      rcpath="eucarc",
                      xof=False,
                      timepergig=1000):
        '''
        Attempts to install and verify an image using the eustore-install command. Creates the command with all the available arguments, and then verifies that
        the appropriate images were installed and registered correctly. 
        help - optional - Boolean - issue eustore command with help flag
        tarball - optional - String - specify local tarball path in which to install from
        description - optional - String - provide description arg to eustore command
        arch= - optional - String provide arch type of image i386 or x86_64
        prefix - optional - String -prefix to use when naming the image, mostly used with tarball option
        bucket= - optional - String - bucket name
        kernel_type -optional -String representing xen or kvm kernel type
        dir - optional -String local directory to unbundle image
        kernel - optional - String eki of installed kernel image to use instead
        ramdisk - optional - string eri of installed ramdisk image to use instead
        url - optional - overide url with this option
        region - optional - String, Name of the region to connect to
        access_key - optional - String, Override access key value
        secret_key - optional - String, Override secret key value
        force - optional - boolean, force installation if image is detected as already installed
        eucarc - optional - boolean, to source eucarc at rc_path
        xof - optional - boolean, to exit on fail
        timepergig - optional - time expected per gig used to determine timeouts
        '''
        image = eustore_image
        self.cur_image = image
        cmd = ""
        start = time.time()
        
        #Check to see if image has already been installed
        if (not force) and  (self.get_registered_emis(eustore_image) is not None):
            eustore_image.results[EustoreTests.install_test] = TestStatus.success
            raise Exception('Image:"'+str(image.name)+'" is already installed')
       
        timeout = int(image.size*timepergig)
        print "image.size:"+str(image.size)+" and timepergig:"+str(timepergig)+" = timeout:"+str(timeout)
        
        #Build the eustore command...
        if eucarc:
            cmd = "source "+str(rcpath)+" && "
        cmd = cmd + self.url_export_string+" eustore-install-image"+self.get_standard_opts(url=url, region=region, access_key=access_key, secret_key=secret_key)  
        cmd = cmd + " -i "+str(eustore_image.id)
        if help == True:
            cmd = cmd + " -h"
            
        if tarball is not None:
            cmd = cmd + " -t "+ str(tarball)
            
        if description is not None:
            cmd = cmd + " -s "+ str(description)
            
        if arch is not None:
            cmd = cmd + " -a "+ str(arch)
            
        if prefix is not None:
            cmd = cmd + " -p "+ str(prefix)
            
        if kernel_type is not None:
            cmd = cmd + " -k "+ str(kernel_type)
        else:
            kernel_type = ""
            
        if bucket is not None:
            cmd = cmd + " -b "+ str(bucket)
        else:
            cmd = cmd + " -b "+ str(eustore_image.name).lower()+"_"+str(kernel_type)
            
        if dir is not None:
            cmd = cmd + " -d "+ str(dir)
            
        if kernel is not None:
            cmd = cmd + " --kernel="+str(kernel)
            
        if ramdisk is not None:
            cmd = cmd + " --ramdisk="+str(ramdisk)
        
        cmd = cmd + " && echo 'SUCCESS'"
        print "-----------------------------------------------------------------------------------------"
        print "-----------------------------------------------------------------------------------------"
        print " image: "+image.name+"_"+str(kernel_type)
        print "-----------------------------------------------------------------------------------------"
        print "-----------------------------------------------------------------------------------------"
        print "Executing cmd:"+str(cmd)
        try:     
            out = self.tester.sys(cmd, timeout=timeout)
            if re.search("SUCCESS",out[len(out)-1]):
                for line in out:
                    if re.search("Error", line):
                        self.debug("Found Error from cmd:"+str(cmd))
                        raise Exception("Found Error from cmd:"+str(cmd))
            else:
                self.debug("Command exited with err:"+str(cmd))
                raise Exception("Command exited with err:"+str(cmd))
            
            
            #Now verify the appropriate images were actually installed on the system
            emi = self.get_registered_emis(eustore_image)[0]
            if (emi is None):
                self.debug('Image:"'+str(image.name)+'" is not found on system after install')
                raise Exception('Image:"'+str(image.name)+'" is not found on system after install')
            
            #Get the associated eki and eri
            eki_id = emi.kernel_id
            eri_id = emi.ramdisk_id
            
            #Verify the appropriate EKI was installed and associated with this EMI
            if (kernel is not None): 
                if(kernel != eki_id):
                    self.debug("Kernel specified does not match specified EKI. EMI:"+str(emi.id)+" Image:"+str(image.name))
                    raise Exception("Kernel specified does not match specified EKI. EMI:"+str(emi.id)+" Image:"+str(image.name)) 
            else:
                eki = self.get_registered_ekis(eustore_image)[0]
                if eki is None:
                    self.debug('EKI for Image:"'+str(image.name)+'" is not found on system after install')
                    raise Exception('EKI for Image:"'+str(image.name)+'" is not found on system after install')
                else:
                    if (eki.id != eki_id):
                        self.debug("Kernel specified does not match associated EKI. EMI:"+str(emi.id)+" Image:"+str(image.name))
                        raise Exception("Kernel specified does not match associated EKI. EMI:"+str(emi.id)+" Image:"+str(image.name))
            self.debug( "Image:"+str(image.name)+" installed kernel:"+str(eki_id))
            #Verify the appropriate ERI was installed and associated with this EMI        
            if (ramdisk is not None):
                if (ramdisk != eri_id):
                    self.debug("Ramdisk specified does not match associated ERI. EMI:"+str(emi.id)+" Image:"+str(image.name))
                    raise Exception("Ramdisk specified does not match associated ERI. EMI:"+str(emi.id)+" Image:"+str(image.name))
            else:
                eri = self.get_registered_eris(eustore_image)[0]
                if eri is None:
                    self.debug('ERI for Image:"'+str(image.name)+'" is not found on system after install')
                    raise Exception('ERI for Image:"'+str(image.name)+'" is not found on system after install')
                else:
                    if (eri.id != eri_id):
                        self.debug("Ramdisk specified does not match associated ERI. EMI:"+str(emi.id)+" Image:"+str(image.name))
                        raise Exception("Ramdisk specified does not match associated ERI. EMI:"+str(emi.id)+" Image:"+str(image.name))
            image.emi = emi.id
            image.eri = eri_id
            image.eki = eki_id
            
            eustore_image.results[EustoreTests.install_test] = TestStatus.success
            self.debug(cmd+" Succeeded")  
        except Exception, e:
            eustore_image.results[EustoreTests.install_test] = TestStatus.failed
            self.debug("Image:"+image.name+" failed to install, err:"+str(e))
            if xof:
                raise Exception("Image:"+image.name+" failed to install, err:"+str(e))
        
        finally:
            elapsed = int(time.time()-start)
            self.debug("Command finish in: "+str(elapsed)+" seconds")
            self.cur_image = None
        
    
            
    def get_standard_opts(self,url=None,region=None,access_key=None,secret_key=None):    
        '''
        Helper method to create the standard options string(s) used when executing a command without sourcing
        eucarc
        '''
        ret = ""
        if region is not None:
            ret = ret+"--region="+str(region)+" "  
        if access_key is not None:
            ret = ret+"-I "+str(access_key)+" "
        if secret_key is not None:   
            ret = ret+"-S "+str(secret_key)+" "
        return ret
        
    
    def get_eustore_image_by_string(self, searchstring, list=None, searchall=False):
        '''
        Attempts to return a list of eustore images by matching the given string to any string in the eustore describe images output
        This output is stored in the eustoreimage.remotestring. 
        '''

        retlist = []
        if list is None:
            list = self.list
        for image in list: 
            if searchall:
                if re.search(searchstring, it.tester.get_all_attributes(image, verbose=False)):
                    retlist.append(image)
            if re.search(searchstring, image.remotestring) or re.search(searchstring,image.id):
                retlist.append(image)
        return retlist
        
    
    def install_all_images(self, hyper="both",eucarc=True, searchstring=None, imglist = None, force=False):
        '''
        Attempts to install all images either in self.list or the provided imglist. 
        By default will install both xen and kvm images, but this can be limited by use of the hyper var.
        Options:
        hyper -optional string - kvm, xen, both. Used to determine which image to use for installation
        eucarc - optional- used to determine whether or not the command should source the eucarc or specify args instead
        searchstring - optional-  will only install images whos remote string matches the given string
        imglist -optional - defaults to self.list, list used to gather all images to be installed
        force - optional - if True, will attempt to install if existing image is detected. Otherwise will skip installed images.
        '''
        if imglist is None:
            imglist = self.list
        if searchstring is not None:
            imglist = self.get_eustore_image_by_string(searchstring, list=imglist)
        for image in imglist:
            try: 
                if (image.kerneltype == "Single"):
                    self.debug("Installing image:"+image.name+" kernel_type:"+image.kerneltype+" , HYPER:NONE")
                    self.install_image(image, eucarc=eucarc)
                else:
                    if (hyper == "both" ) or (hyper == "xen"):
                        self.debug("Installing image:"+image.name+" kernel_type:"+image.kerneltype+" HYPER:XEN")
                        self.install_image(image, eucarc=True, kernel_type="xen")
                    if (hyper == "both ") or (hyper == "kvm"):
                        self.debug("Installing image:"+image.name+" kernel_type:"+image.kerneltype+" HYPER:KVM")
                        self.install_image(image, eucarc=True, kernel_type="kvm")      
            except Exception, ie:
                self.debug("Image;"+image.name+" failed."+str(ie))
                pass
    
    def get_eustore_image_by_id(self, emi_id=None, eri_id=None, eki_id=None):
        '''
        Returns a list of eustore images which match the provided emi,eri,eki ids of the installed images
        '''
        
        
        for image in self.list:
            if (emi_id is not None) and (not re.search(emi_id, image.emi)):      
                continue  
            if (eki_id is not None) and (not re.search(eki_id, image.eki)):      
                continue
            if (eri_id is not None) and (not re.search(eri_id, image.eri)):      
                continue
            self.debug("get_eustore_image_by_id,emi_id="+str(emi_id)+" = "+str(image.name))
            return image
        return None
        
        
        
    def sync_installed_list(self, updateimage=True):
        '''
        Attempts to get a diff of images in our list vs what the system is reporting. 
        This relies upon the bucket location having the image id text in it. 
        Returns a list of images found in our eustore list that are not believed 
        to be installed on the system. Can be later passed to install_all_images, etc..
        '''
        retlist = []
        for image in self.list:
            self.debug("Checking to see if image:"+str(image.name)+" is installed")
            emis = self.get_registered_emis(image)
            if emis is None:
                self.debug("Image: "+str(image.name)+" not found on system")
                retlist.append(image)
            else:
                emi = emis[0]
                self.debug("Image: "+str(image.name)+" found installed on system")
                if updateimage:
                    #Get the associated eki and eri
                    image.eki = emi.kernel_id
                    image.eri = emi.ramdisk_id
                    image.emi = emi.id
                    image.results[EustoreTests.install_test]=TestStatus.success
        return retlist
    
    def print_image_list(self,list=None, verbose=False):
        '''
        Traverses a list of images and displays the image information and current test results status.
        '''
        if list is None:
            list = self.list
        for image in list:
            self.debug("IMAGE: "+str(image.name)+" Install Status: "+str(image.results[EustoreTests.install_test]))
            if verbose:
                image.printdata()
                    
    def test_image_list(self, image_list=None, vmtype= None, zone=None, userlist=[], rootpass=None, xof=False,volcount=2):
        '''
        Attempts to traverse a list of images and run the image test suite against them
        '''
        if image_list is None:
            image_list = self.list
        if zone is None:
            zone = self.zone    
        for image in image_list:
            try:
                self.run_image_test_suite(image, vmtype=None, zone=zone, userlist=[], rootpass=None, xof=False,volcount=2)
                time.sleep(10)
            except Exception, e:
                self.debug("Caught Exception while running image test for:"+str(image.name)+", err:"+str(e))
                pass
            

            
                
                
    def run_image_test_suite(self,image, vmtype=None, zone='PARTI00', userlist=[], rootpass=None, xof=False, volcount=2):  
        '''
        Runs a set of tests against an image, starts by running an euinstance of an image. If the image continues
        to running the remaining tests are ran against the image. Logging/results/debugging messages should be printed to the
        image specific dir/file. usually ../artifacts/*imagename*.log
        
        '''
        self.cur_image = image #set logger for this image    
        image.clearresults()
        failcode = 0 
        
        try:
            #first try to run the image, exit if this fails as there's no point in running the remaining tests
            try:
                res = self.run_image(image, vmtype=vmtype, zone=zone)
                inst = res.instances[0]
            except Exception, re:
                image.results[EustoreTests.running_test] = TestStatus.failed
                self.debug("("+str(image.name)+") error:"+str(re))
                raise re
                return
            
            try:
                self.instance_ssh_test(inst, image=image)
                self.debug("SUCCESS - SSH TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - SSH TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                raise e
                return
            
            #Perform tests on the running instance, continue on failure for these tests
            try:
                self.instance_metadata_test(inst, image=image)
                self.debug("SUCCESS - METADATA TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - METADATA TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            try:
                self.instance_root_test(inst, image=image)
                self.debug("SUCCESS - ROOT TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - ROOT TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            try:
                self.instance_users_test(inst, userlist=userlist, image=image)
                self.debug("SUCCESS - USERS TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - USERS TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            try:
                self.instance_zeroconf_test(inst, image=image)
                self.debug("SUCCESS - ZEROCONF TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - ZEROCONF TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            try:
                self.instance_virtio_present_test(inst, image=image)
                self.debug("SUCCESS - VIRTIO PRESENT TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - VIRTIO PRESENT TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            try:    
                self.instance_attach_vol_test(inst,volcount=volcount, image=image)
                self.debug("SUCCESS - ATTACH VOLUMES TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - ATTACH VOLUME TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            try:
                self.instance_reboot_test(inst, image=image)
                self.debug("SUCCESS - REBOOT TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - REBOOT TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            try:
                self.instance_detach_volumes_test(inst, count=volcount, image=image)
                self.debug("SUCCESS - DETACH VOLUMES TEST - ("+str(image.name)+")")
            except Exception, e:
                self.debug("!!!!!! FAILED - DETACH VOLUMES TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
                
            try:
                self.instance_terminate_test(res, image=image)
                self.debug("SUCCESS - TERMINATE TEST - ("+str(image.name)+")")
            except Exception, e: 
                self.debug("!!!!!! FAILED -  TEST - ("+str(image.name)+") error:"+str(e))
                failcode = 1 
                pass
            
            
        except Exception, e:
            self.debug("!!!!!! FAILED  - ("+str(image.name)+") error:"+str(e))
            failcode = 1
            raise e
        finally:
            self.clean_up_running_instances_for_image(image)
            
            self.debug(str('FAIL' if failcode == 1 else "SUCCESS")+" IMAGE:"+str(image.name))
            image.printdata(printmethod=self.debug)
            self.cur_image = None
            return failcode
        
            
    def clean_up_running_instances_for_image(self,image, timeout=300):
        '''
        Additional cleanup mechanism to help remove instances from failed terminate tests
        image - optional - eustoreimage object to report results against
        timeout - optional - seconds to wait on cleanup before failure
        
        '''
        self.debug("clean_up_running_instance_for_image: "+str(image.name))
        failstr=''
        emis = self.get_registered_emis(image)
        if emis != []:
            emi = emis[0]
            for instance in self.tester.get_instances(image_id=emi.id):
                try:
                    if (instance.state != 'terminated'):
                        self.debug("Found Instance to terminate:"+str(instance.id))
                        instance.terminate()
                        self.tester.wait_for_instance(instance, state='terminated')
                    self.debug("Instance:"+str(instance.id)+" terminated")
                except Exception, e:
                    failstr = failstr+ "instance:"+instance.id+"Failed:"+str(e)+"\n"
                    pass
        else:
            self.debug("No instances found to clean up")
        if failstr != '':
            raise Exception("Failed to terminated instances:"+str(failstr))
        
                
            
    def run_image(self,image, username=None, password=None, vmtype=None, zone='PARTI00'):
        '''
        Attempts to verify that an instance from the provided image can be ran, by
        default an ssh session will be created to this sesssion. If the instance does not progress
        to running state within a given timeout, or if the ssh connection can not be established, the 
        test will fail. 
        inst - mandatory - euinstance object
        username - optional- username to use for ssh login for this instance
        password - optional - password to user for ssh login for this instance (key'd login by default)
        vmtype - optional - string representing vmtype, otherwise test will attempt to determine vmtype based on eustoreimage size
        zone - optional - string representing availability zone to launch image in
        image - optional - eustoreimage object to report results against
        '''
        
        if vmtype is None:
            if image.size <= 2:
                vmtype = 'c1.medium'
            elif image.size <= 5:
                vmtype = 'm1.large'
            else:
                vmtype = 'm1.xlarge'
        self.debug("#####STARTING run_image test########")
        image.results[EustoreTests.running_test] = TestStatus.failed
        if username is None:
            username = str(image.loginuser)
        try:
            emi = self.get_registered_emis(image)[0]
        except Exception, e:
            raise Exception("Could not get registered EMI for image:"+str(image)+" err:"+str(e))
        
        self.debug("Running image:"+str(image)+", vmtype:"+str(vmtype)+" keypair:"+str(self.keypair.name)+" group:"+str(self.group)+" zone:"+str(zone))
        res = self.tester.run_instance(image=emi, keypair=self.keypair.name, group=self.group, type=vmtype, zone=zone,username=username, password=password)
        if res == []:          
            raise Exception("Reservation was empty, failed to run: "+str(image))      
        image.results[EustoreTests.running_test] = TestStatus.success
        return res
        
        
    def instance_ssh_test(self,inst,image=None):
        '''
        This test attempts to issue a remote command via the euinstance ssh session and
        verify the  output
        inst - mandatory - euinstance object
        image - optional - eustoreimage object to report results against
        '''
        self.debug("################STARTING instance_ssh_test #####################")
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image,EustoreTests.ssh_test, TestStatus.failed )
        try:
            if inst.found("echo 'good'", "^good"):
               self.set_result(image,EustoreTests.ssh_test, TestStatus.success)
               return
            else:
                raise Exception("Expected text not returned from remote ssh command")
        except Exception, e:
            raise Exception("("+str(image)+") Failed ssh command test:"+str(inst.id)+"err:"+str(e))
            
        
            
    def instance_metadata_test(self,inst,image=None):
        '''
        This test attempts to verify the basic functionality of Metadata
        on this instance
        inst - mandatory - euinstance object
        image - optional - eustoreimage object to report results against
        '''
        self.debug("#######STARTING instance_metadata_test##########")
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.metadata_test, TestStatus.failed)
       
        if inst.get_metadata('instance-id')[0] != inst.id:
            raise Exception("("+str(image)+") Metadata test failed")
        self.set_result(image, EustoreTests.metadata_test, TestStatus.success)
        
        
    def instance_virtio_present_test(self,inst,image=None):
        '''
        This test attempts to verify that the instance has the virtio kernel modules available. 
        It does not check if they are installed per NC configuration. 
        inst - mandatory - euinstance object
        image - optional - eustoreimage object to report results against
        '''
        self.debug("#######STARTING instance_virtio_present_test##########")
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.virtiopresent_test, TestStatus.failed)
        #look for filename line in modinfo...
        #filename:       /lib/modules/2.6.28-11-generic/kernel/drivers/virtio/virtio.ko
        out = inst.sys('modinfo virtio')
        for line in out:
            if (re.search("^filename", line) and re.search("virtio.ko", line)):
                self.set_result(image, EustoreTests.virtiopresent_test, TestStatus.success) 
                return          
        raise Exception("("+str(image)+") virtio present test failed")
               
   
        
    def instance_attach_vol_test(self,inst, volcount=1, recycle=True, image=None, vol_list=[], timepergig=180):
        '''
        This test attempt to verify attachment of volume(s) to an instance
        inst - mandatory - euinstance object
        volcount - optional - number of volumes to attach to instance if a vol_list is not provided
        recycle - boolean - first attempt to use existing volumes for this test rather create new ones
        image - optional - eustoreimage object to report results against
        vol_list -optional -list of euvolumes to be attached to instance
        '''

        self.debug("###########STARTING instance_attach_vol_test##########")
        vols = 0
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.attach_volume_test, TestStatus.failed)
    
        zone = inst.placement
        try:
            if vol_list != []:
                for vol in vol_list and vols <= volcount:
                    inst.attach_volume(vol)
                    vols += 1 
            else:
                for i in xrange(0,volcount):
                    if recycle:
                        try:
                            vol = self.tester.get_volume(status='available', zone=zone, maxsize=1)
                            self.debug('Using recycled and available:'+str(vol))
                        except Exception, e:
                            self.debug("Err trying to get an existing volume, making new one... err:"+str(e))
                            vol = self.tester.create_volume(zone, timepergig=timepergig)
                            pass
                    else:
                        elf.debug("Recycle vols not set, created new")      
                        vol = self.tester.create_volume(zone)              
                    timeout= vol.size * timepergig
                    inst.attach_volume(vol,timeout=timeout)
        except Exception, ae:
            raise Exception("("+str(image)+") Failed to attach volume to:"+str(inst.id)+"err:"+str(ae))
        self.set_result(image, EustoreTests.attach_volume_test, TestStatus.success)
        
    
    def instance_detach_volumes_test(self, inst, image=None, count=0, timeout=60):
        '''
        Attempts to verify 'count' number of volumes detach correctly from an instance
        inst - mandatory - euinstance object
        image - optional - eustoreimage object to report results against
        count - optional - integer, # of volumes to detach from instance. 0 = all
        timeout - optional - integer. # of seconds to wait for volumes to detach before failure
        '''
        self.debug("#############STARTING instance_detach_volume_test#############")
       
        attached_count = len(inst.attached_vols)
        if attached_count == 0:
            self.debug("("+str(image)+")No attached volumes detected, skipping detach volume test:"+str(inst.id))
            self.set_result(image, EustoreTests.detach_volume_test, TestStatus.not_run)
            return
        if count == 0:
            count = attached_count
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.detach_volume_test, TestStatus.failed)
       
        try:
            detached = 0
            for evol in inst.attached_vols:
                inst.detach_euvolume(evol, timeout=timeout)
                detached += 1
                if detached >= count:
                    break
        except Exception, e:
            raise Exception("("+str(image)+") Failed to detattach volumes:"+str(inst.id)+"err:"+str(e))
        self.set_result(image, EustoreTests.detach_volume_test, TestStatus.success)
        
    
    def instance_reboot_test(self,inst, checkvolstatus=True, image=None):
        '''
        This attempt to reboot an instance, if volumes were present before the reboot, this test
        will attempt to verify that those volumes are intact as well as verify data integrity with md5
        inst - mandatory - euinstance object
        image - optional - eustoreimage object to report results against
        checkvolstatus - boolean - flag to confirm volume attached states post reboot
        
        '''
        
        self.debug("############STARTING instance_reboot_test#############")
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.reboot_test, TestStatus.failed)
        
        try:
            inst.reboot_instance_and_verify(checkvolstatus=checkvolstatus)
            self.set_result(image, EustoreTests.reboot_test, TestStatus.success)
            return
        except Exception, e:
            self.set_result(image, EustoreTests.reboot_test, TestStatus.failed)
            raise Exception("("+str(image)+")Failed instance_reboot_test, err:"+str(e))
        
        
    def instance_users_test(self, inst, userlist=[], image=None):
        '''
        This attempts to verify that no nomral (non-system) users are present on 
        the instance/image.
        inst - mandatory - euinstance object
        userlist - optional - list strings representing user names  expected to be found on instance. 
        image - optional - eustoreimage object to report results against
        
        '''
        self.debug("############STARTING instance_users_test#################")
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.user_test, TestStatus.failed)
       
        users = inst.get_users() 
        if users != userlist:
            raise Exception("("+str(image)+")Image has unknown Non-system users: "+"".join(users))
        self.set_result(image, EustoreTests.user_test, TestStatus.success)
        
        
        
    def instance_zeroconf_test(self, inst, netfile='/etc/sysconfig/network', image=None):
        '''
        This test attempts to verify the instance is not using zeroconf network configuration. 
        The zeroconf config may conflict with certain network modes.
        Note: netfile set to null only checks routing table for 169.254.0.0 addr
        inst - mandatory - euinstance object
        netfile - optional -string representing file path to network config
        image - optional - eustoreimage object to report results against
        '''
        self.debug("############STARTING instance_zeroconf_test#################")
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.zeroconf_test, TestStatus.failed)
            
        if netfile is not None:
            #First verify netfile exists
            try:
                out = inst.sys("ls "+str(netfile))[0].strip()
                if ( out != netfile ):
                    raise Exception(str(out) +"!=" +  str(netfile))
            except Exception, e:
                raise Exception("Could not stat:"+str(netfile)+", err:"+str(e))
            
            #Verify the network file has zeroconf disabled
            cmd = "cat "+str(netfile)+" | grep NOZEROCONF | grep -v '^#'"         
            if inst.found(cmd,"NOZEROCONF"):
                self.debug("Found NOZEROCONF in:"+str(netfile))
            else:
                raise Exception("NOZEROCONF not found, zero conf is not disabled")
        
        if inst.found('route -n', '169.254.0.0'):
            raise Exception("ZEROCONF network detected in instance")
        self.set_result(image, EustoreTests.zeroconf_test, TestStatus.success)
        

        
    def instance_root_test(self, inst, password = None, image=None):
        '''
        Attempts to verify whether the root user on the instance has a password or not. 
        A password can be specififed if it is expected, by default it expects no password
        inst - mandatory - euinstance object
        password - optional - string representing an expected root user password
        image - optional - eustoreimage object to report results against
        '''
        self.debug("############STARTING isntance_root_test################")
        if image is None:
            image = self.get_eustore_image_by_id(emi_id=inst.image_id)
        self.set_result(image, EustoreTests.root_test, TestStatus.failed)
        ipassword = inst.get_user_password('root') 
        if ipassword != password:
            self.set_result(image, EustoreTests.root_test, TestStatus.failed)
            raise Exception("("+str(image)+")root user has bad password! found:"+str(ipassword)+"!="+str(password))
        self.set_result(image, EustoreTests.root_test, TestStatus.success)
       
        
        
 
        
    def instance_terminate_test(self,res,image=None):
        '''
        Attempts to confirm that an image terminates within a certain timeout
        res - mandatory - reservation of instance(s) to terminate
        image - optional - eustoreimage object to report results against
        '''
        self.debug("#######STARTING instance_terminate_test##########")
        if res.instances == []:
            self.debug('Reservation was empty in instance_terminate_test. Not running')
            self.set_result(image, EustoreTests.terminate_test, TestStatus.not_run)
            return
        try:
            if image is None:
                inst = res.instances[0]
                image = self.get_eustore_image_by_id(emi_id=inst.image_id)
            self.set_result(image, EustoreTests.terminate_test, TestStatus.failed)
            self.tester.terminate_instances(res)
            self.set_result(image, EustoreTests.terminate_test, TestStatus.success)
        except Exception, e:
            self.debug("Trying single Termination, Problem with terminating reservation:" +str(e))
            try:
                self.clean_up_running_instances_for_image(image)
                self.set_result(image, EustoreTests.terminate_test, TestStatus.success)
                return
            except Exception, se:
                self.debug("Failed single termination for image:"+str(image))
                raise se
                

    def get_images_by_test_result(self,list=None, eustoretest=None,result=TestStatus.failed ):
        """ 
        Attempts to return a subset list of images who's test results match the result give. 
        By default this will traverse each image in self.list, will then traverse the image.results list for each
        EustoreTest matching TestStatus.failed.
        list - optional - list of eustoreimages to search within
        eustoretest - optional - eustore test used in search criteria
        result - optional - TestStatus type result to search for
        """
        retlist = []
        if list is None:
            list = self.list
        
        if eustoretest is None:
            #return all images who have any test which results matches 'result'
            for image in list:
                for test in image.results:
                    if image.results[test] == result:
                        retlist.append(image)
            return retlist
        else:
            #return any images whos specific eustore test mataches 'result'
            for image in list:
                try:
                    t = EustoreTests.__dict__[eustoretest]
                except Exception, te:
                    raise Exception("get_images_by_test_result, provided invalid test type:"+str(eustoretest)+", err;"+str(te))
                if image.results[t] == result:
                    retlist.append(image)
            return retlist
        
    
    
        
    def set_result(self,image,test,status):
        '''
        convience method to apply test results to an image
        '''
        #if image is none do nothing
        if image is None:
            return
        self.validate_eustore_test(test)
        self.validate_eustore_status(status)
        image.results[test] = status
        
    def validate_eustore_test(self,test):
        '''
        Validate a given test is defined as a eustore test
        '''
        for k in EustoreTests.__dict__:
            if test == EustoreTests.__dict__[k]:
                return 
        raise Exception("Test '"+str(test)+"' is  not defined in EustoreTests")
    
    def validate_eustore_status(self, status):
        '''
        Validate a given status is defined
        '''
        for k in TestStatus.__dict__:
            if status == TestStatus.__dict__[k]:
                return 
        raise Exception("Status '"+str(status)+"' is not defined in TestStatus")                        
                
    
    
    def print_image_list_results(self,list=None):
        '''
        prints the image data and results for a list of eustore images. Defaults to current eustoretestsuite's list
        list - optional - list of eustore images
        '''
        if list is None:
            list = self.list
        for image in list:
            self.cur_image = image
            image.printdata()
            self.cur_image = None
        
    
            
            
            


#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#


#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#
#    "running_test" - verify image-instance goes to running 
#    "metadata_test" - verify basic cloud metadata service can be used from this image-instance
#    "user_test" = verify that only login users provided in userlist exist on this image-instance
#    "root_test" - verify that a root password is not set, unless one is given
#    "attach_volume_test" - verify volcount number of volumes can be attached, and appear within this instance
#    "detach_volume_test" - verify attached volumes can be detached, guest dev is removed
#    "reboot_test" - verify instance can be rebooted, ssh re-established, any volumes remain attached
#    "terminate_test" - verify instance goes to proper instance state
#    "ssh_test" - verify an ssh session can be established with this instance once running
#    "zeroconf_test"= verify zero conf is disaabled on the image
#    "virtiopresent_test" - verify that the virtio mods are available on this image
#
#    @author: clarkmatthew

import unittest
from eutester.eutestcase import EutesterTestCase
from eutester.eutestcase import EutesterTestResult
from eustoretestsuite import Eustoretestsuite
import os
import argparse

testsuite = None
zone = None
config_file = None
password = None
credpath = None
keypair = None
group = None
emi = None
image = None
vmtype = None
userlist = None
xof = None
volcount = None
rootpass = None


class emi_tests(unittest.TestCase):
    
    def setUp(self):
        '''
        Main function of setup is to instantiate an Eustore Test Suite object. With command line args. 
        '''
        testsuite = self.testsuite = Eustoretestsuite( config_file=config_file, 
                                           password=password,
                                           keypair=keypair, 
                                           group=group,  
                                           zone=zone,
                                           credpath=credpath )
        
    def test_emi_suite(self):
        '''
        Full suite of image specific tests
        '''
        testsuite = self.testsuite
        images = testsuite.get_system_images(emi = emi)
        if len(images) != 1:
            raise exception('('+str(len(images))+') != (1) image matches for emi string:'+str(emi) )
        image = self.image = testsuite.convert_image_list_to_eustore_images(images)[0]
        self.testsuite.run_image_test_suite(image, vmtype=vmtype, zone=zone, userlist=userlist, rootpass=rootpass, xof=xof, volcount=volcount)
        
        
    def tearDown(self):
        '''
        Clean up resource created during test, output results
        '''
        try:
            self.testsuite.clean_up_running_instances_for_image(image)
        except:
            self.testsuite.debug("Cleanup failed. Exiting normally")
        finally:
            #print the image test results
            self.image.printdata()
            
        
            
    


if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch
    parser = argparse.ArgumentParser(prog="emi_basic_test.py",
                                     version="Test Case [emi_basic_test.py] Version 0.1",
                                     description="Attempts to tests and provide info on specific areas of a\
                                     a given registered image.",
                                     usage="%(prog)s --emi  --credpath <path to creds> <...options> ")
    
    
    
    parser.add_argument('--emi', 
                        help="pre-installed emi id which to execute these tests against", default=None)
    parser.add_argument('--credpath', 
                        help="path to credentials", default=None)
    parser.add_argument('--zone', 
                        help="zone to use in this test, defaults to testing all zones", default=None)
    parser.add_argument('--password', 
                        help="password to use for machine root ssh access", default='foobar')
    parser.add_argument('--keypair', 
                        help="keypair to use when launching instances within the test", default=None)
    parser.add_argument('--group', 
                        help="group to use when launching instances within the test", default=None) 
    parser.add_argument('--config',
                       help='path to config file', default=None) 
    parser.add_argument('--vmtype',
                       help='vmtype to run this image with', default=None) 
    parser.add_argument('--rootpass',
                       help='root password this image is expected to be using', default=None) 
    parser.add_argument('--volcount',
                       help='number of volumes to use in test, default is 2', default=2) 
    parser.add_argument('--xof',
                       help='exit on fail boolean. Default is false, to continue testing', default=False) 
    parser.add_argument('--userlist', nargs='+', 
                        help="Login users expected to be found on this image", 
                        default= [])
    parser.add_argument('--tests', nargs='+', 
                        help="test cases to be executed", 
                        default= ['test_emi_suite'])
    
    args = parser.parse_args()
    
    '''
    Assign parsed arguments to this testcase globals
    '''
    if args.config is not None:
        #if file was not provided or is not found
        if not os.path.exists(args.config):
            print "Error: Mandatory Config File '"+str(args.config)+"' not found."
            parser.print_help()
            exit(1)
    elif args.credpath is None:
        print "Error either config file or credpath needs to be provided"
        exit(1)
    zone = args.zone
    config_file = args.config
    password = args.password
    credpath = args.credpath
    keypair = args.keypair
    group = args.group
    emi = args.emi
    userlist = args.userlist
    xof = args.xof
    volcount = args.volcount
    rootpass = args.rootpass
    vmtype = args.vmtype
    
    for test in args.tests:
        result = unittest.TextTestRunner(verbosity=2).run( emi_tests(test))
        if result.wasSuccessful():
            pass
        else:
            exit(1)

    
  


#!/usr/bin/python

import time
from eucaops import Eucaops
from eucaops import EC2ops
from eutester.eutestcase import EutesterTestCase
import os
import random

class PopulateUpgrade(EutesterTestCase):
    def __init__(self, extra_args= None):
        self.setuptestcase()
        self.setup_parser()
        if extra_args:
            for arg in extra_args:
                self.parser.add_argument(arg)
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( credpath=self.args.credpath, config_file=self.args.config,password=self.args.password)
        self.tester.poll_count = 120

        self.security_groups = []

        ### Generate a keypair for the instance
        self.keypair = self.tester.add_keypair( "keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)
        self.image = self.args.emi
        if not self.image:
            self.image = self.tester.get_emi(root_device_type="instance-store")
        self.address = None
        self.volume = None
        self.snapshot = None
        self.private_addressing = False
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name
        self.reservation = None

    def clean_method(self):
        pass

    def Instances(self, type="instance-store"):
        """
        This case was developed to run through a series of basic instance tests.
             The tests are as follows:
                   - execute run_instances command
                   - make sure that public DNS name and private IP aren't the same
                       (This is for Managed/Managed-NOVLAN networking modes)
                   - test to see if instance is ping-able
                   - test to make sure that instance is accessible via ssh
                       (ssh into instance and run basic ls command)
             If any of these tests fail, the test case will error out, logging the results.
        """
        test_image = self.tester.get_emi(root_device_type=type)

        ### Add and authorize a group for the instance
        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )

        self.security_groups.append(self.group)

        # Test: INSTANCESTORE VOLATTACH:no ADDR:user
        instance_1 = self.tester.run_instance(test_image, keypair=self.keypair.name, group=self.group.name).instances[0]
        instance_1_address = self.tester.allocate_address()
        self.tester.associate_address(instance=instance_1, address=instance_1_address)

        # Test: INSTANCESTORE VOLATTACH:no ADDR:system
        instance_2 = self.tester.run_instance(test_image, keypair=self.keypair.name, group=self.group.name).instances[0]

        # Test: INSTANCESTORE VOLATTACH:no ADDR:system
        instance_3 = self.tester.run_instance(test_image, group=self.group.name, private_addressing=True, is_reachable=False).instances[0]

        # Test: INSTANCESTORE VOLATTACH:yes ADDR:user
        instance_4 = self.tester.run_instance(test_image, keypair=self.keypair.name, group=self.group.name).instances[0]
        instance_4_address = self.tester.allocate_address()
        self.tester.associate_address(instance=instance_4, address=instance_4_address)
        volume = self.tester.create_volume(zone=self.zone)
        instance_4.attach_volume(volume=volume)

        # Test: INSTANCESTORE VOLATTACH:yes ADDR:system
        instance_5 = self.tester.run_instance(test_image, keypair=self.keypair.name, group=self.group.name).instances[0]
        volume = self.tester.create_volume(zone=self.zone)
        instance_5.attach_volume(volume=volume)

        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )
        self.security_groups.append(self.group)
        # Test: INSTANCESTORE VOLATTACH:yes ADDR:system
        instance_6 = self.tester.run_instance(test_image, group=self.group.name, private_addressing=True, is_reachable=False).instances[0]

    def PopulateAll(self):
        self.Instances("instance-store")
        self.Instances("ebs")

if __name__ == "__main__":
    testcase = PopulateUpgrade()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or [ "PopulateAll"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)